*D0-FILE TO ACCOMPANY EXPERIMETRICS BOOK

* TO ENSURE THAT THIS DO-FILE RUNS UNINTERRUPTED, YOU NEED TO CARRY OUT THE FOLLOWING TWO TASKS:

* USE THE COMMAND cd "drive\folder_name" TO CHANGE THE WORKING FOLDER TO
* THE FOLDER IN WHICH THE DATA SETS ARE STORED

* INSTALL THE FOUR USER-WRITTEN COMMANDS: escftest; cdfplot; mdraws; fmm
* TO INSTALL THEM, START BY USING findit command_name

clear
set more off
prog drop _all

*CHAPTER 3: TREATMENT TESTING

* 3.3.1 THE BINOMIAL TEST

use lottery_choice_sim

tab y
bitest y=0.5

* 3.3.2 FISHER'S EXACT TEST

tab y male, col
tab y male, col exact

* 3.3.3 THE CHI-SQUARED TEST

tab y male, col chi2

* 3.3.4 THE CHI-SQUARED TEST ON A REAL DATA SET

use holdup, clear
set more off

tab invest treatment, col chi2

tab invest treatment if treatment!=1, col chi2

* 3.4 TESTING FOR NORMALITY

hist offer, disc freq normal xline(60) xlabel(0(10)100)

sktest offer

sktest offer if treatment==1

sktest offer if treatment==2

sktest offer if treatment==3

swilk offer

* 3.5 TREATMENT TESTING

* 3.5.1 PARAMETRIC TESTS OF TREATMENT EFFECTS

table treatment, contents(n offer mean offer)

ttest offer if treatment!=2, by(treatment)

ttest offer if treatment!=2, by(treatment) unequal

sdtest offer if treatment!=2, by(treatment)

* 3.5.2 NON-PARAMETRIC TESTS OF TREATMENT EFFECTS: THE MANN-WHITNEY TEST

ranksum offer if treatment!=2, by(treatment)

* 3.5.3 THE BOOTSTRAP

bootstrap t=r(t), rep(999) nodrop : /// 
ttest offer if treatment!=2, by(treatment) 

*NEXT COMMAND TAKES QUITE A LONG TIME TO EXECUTE

bootstrap t=r(t), nodots rep(9999) nodrop saving("hello.dta", replace) : ///
ttest offer if treatment!=2, by(treatment) 

* EXAMINE CONTENTS OF DATA SET "hello" SAVED IN LAST BOOTSTRAP COMMAND:

use hello
summ t 
hist t

erase hello.dta

* 3.5.4 TESTS COMPARING ENTIRE DISTRIBUTIONS

use forsythe, clear

ksmirnov y, by(dic_ult)

cdfplot y, by(dic_ult)

* For the Epps-Singleton test, you first need to download the escftest command

escftest y, group(dic_ult)

* 3.6 TESTING FOR GENDER EFFECTS

use ug_sim

hist y if male_i==0
hist y if male_i==1

sktest y if male_i==0
sktest y if male_i==1

sdtest y, by(male_i)

ttest y, by(male_i)
ttest y, by(male_i) unequal

ranksum y, by(male_i)


* 3.7.3 WITHIN-SUBJECT TESTS: CONTINUOUS OUTCOMES

use give_take_sim

scatter y2 y1, msize(1) jitter(2) yline(0) xlabel(0(1)5) ylabel(-5(1)5)

ttest y2=y1

signrank y2=y1

signtest y2=y1

*  CHAPTER 4: THEORY TESTING, REGRESSION, AND DEPENDENCE

* 4.2.3 THE SIMULATED AUCTION DATA

use common_value_sim, clear

table session, contents(n y mean y)

sort session period market i

* 4.3 TESTS OF AUCTION THEORY

* USE ONLY FIRST 4 SESSIONS (TO FIX N=4 AND EPSILON=6)

keep if session<5

* TEST OF RNNE

ttest y=6

* TEST OF NAIVE BEHAVIOUR

ttest y=0

* 4.4 TESTS OF COMPARATIVE STATIC PREDICTIONS

* READ DATA AGAIN

use common_value_sim, clear

*USE ONLY FIRST 8 EXPERIMENTS (TO FIX EPSILON)

keep if session<9

* MANN-WHITNEY TEST FOR EFFECT OF NUMBER OF BIDDERS

ranksum y, by(N6)

* T-TEST FOR EFFECT OF NUMBER OF BIDDERS

ttest y, by(N6)

* T-TEST USING REGRESSION

regress y N6

* T-TEST ACCOUNTING FOR DEPENDENCE

regress y N6, vce(cluster i)

* T-TEST ACCOUNTING FOR DEPENDENCE USING BLOCK BOOTSTRAP

reg y N6, vce(bootstrap, rep(999) cluster( i) )

* 4.5 MULTIPLE REGRESSION WITH COMPLETE DATA SET

* READ DATA AGAIN

use common_value_sim, clear
set more off

* INTRODUCING THE EFFECT OF UNCERTAINTY

regress bid x N6 eps24 N6eps24, vce(cluster i)

* TEST OF RNNE

test (x=1) (_cons=-6) (N6=-2) (eps24=-6) (N6eps24=-2)

* INTRODUCING THE EFFECT OF EXPERIENCE

regress bid x rec_period N6 eps24 N6eps24, vce(cluster i)
test (x=1) (_cons=-6) (N6=-2) (eps24=-6) (N6eps24=-2)

* INTRODUCING THE EFFECT OF CASH BALANCE

regress bid x balance rec_period N6 eps24 N6eps24, vce(cluster i)
est store ols
test (x=1) (_cons=-6) (N6=-2) (eps24=-6) (N6eps24=-2)

* 4.6 PANEL DATA ESTIMATION

xtset i period
xtreg bid x balance rec_period N6 eps24 N6eps24 , re
test (x=1) (_cons=-6) (N6=-2) (eps24=-6) (N6eps24=-2)

* 4.7 MULTI-LEVEL MODELLING

* ONE-LEVEL IS EQUIVALENT TO LINEAR REGRESSION (POOLED OLS)

xtmixed bid x balance rec_period N6 eps24 N6eps24  
est store one_level

* TWO-LEVEL IS EQUIVALENT TO RANDOM EFFECTS

xtmixed bid x balance rec_period N6 eps24 N6eps24  || i: 
est store two_level
test (x=1) (_cons=-6) (N6=-2) (eps24=-6) (N6eps24=-2)

* LR TEST OF RANDOM EFFECTS VERSUS POOLED OLS

lrtest two_level one_level

* THREE-LEVEL: CLUSTERING AT INDIVIDUAL AND SESSION LEVELS

xtmixed bid x balance rec_period N6 eps24 N6eps24  || session: || i: 
est store three_level
lrtest three_level two_level
test (x=1) (_cons=-6) (N6=-2) (eps24=-6) (N6eps24=-2)

* THREE LEVEL WITH RANDOM SLOPE ON CASH BALANCE

xtmixed bid x balance rec_period N6 eps24 N6eps24  || session: || i: balance
est store random_slope
test (x=1) (_cons=-6) (N6=-2) (eps24=-6) (N6eps24=-2)

* LR TEST FOR HETEROGENEITY IN CASH BALANCE EFFECT

lrtest random_slope three_level

* 4.8 CONTESTS
 
* READ DATA

use chowdhury, clear 
set more off

* HISTOGRAM OF BIDS IN BASELINE (PL) TREATMENT

hist bid if (p==1)&(l==1)

* GENERATE OVER-BID (o), TREATMENT INTERACTION (sc), AND EXPERIENCE VARIABLE (rec_t)

gen o = bid - 15
gen sc=s*c
gen rec_t=1/t

* DECLARE PANEL

xtset i t

*RANDOM EFFECTS MODEL WITHOUT INTERACTION:

xtreg o s c rec_t

*RANDOM EFFECTS MODEL WITH INTERACTION:

xtreg o s c sc rec_t 

*META ANALYSIS

* READ DATA

use sheremeta, clear

* META REGRESSION 

reg  overbid endowment_rel n partners one_shot

* LOWESS SMOOTHER OF OVERBID AGAINST RELATIVE ENDOWMENT

lowess overbid endowment_rel

* META REGRESSION INCLUDING RELATIVE-ENDOWMENT-SQUARED

reg  overbid endowment_rel end2 n partners one_shot

* COMPUTE "EFFORT-MAXIMISING ENDOWMENT"

nlcom end_star: -_b[endowment_rel]/(2*_b[end2])

* CHAPTER 5:  MODELLING OF DECISION TIMES USING REGRESSION ANALYSIS

use decision_times_sim, clear
set more off

* 5.4 AN ECONOMETRIC MODEL OF EFFORT ALLOCATION

* SUMMARY STATISTICS OF DECISION TIME IN SECONDS

summ dt, detail

* HISTOGRAM OF DECISION TIME

hist dt if dt<40

* DECLARE PANEL, AND OBTAIN TIME SERIES GRAPHS BY SUBJECT

xtset i t
xtline dt if dt<20

* LOWESS SMOOTHER OF LOG(DECISION-TIME) AGAINST POSITION IN SEQUENCE

lowess log_dt tau, msize(0.1) xline(50)

* LOWESS SMOOTHER OF DECISION TIME AGAINST CLOSENESS TO INDIFFERENCE
* (THIS ONE NEEDS TO BE DONE IN 2 STAGES IN ORDER TO RESTRICT THE SCALE OF THE VERTICAL AXIS)

capt drop dt_hat
lowess dt cti, msize(0.1) generate(dt_hat) 
sort cti
twoway (scatter dt cti, msize(0.1)) (line dt_hat cti) if dt<10, ytitle(Decision time in seconds) xtitle(Closeness to indifference)

* 5.5 PANEL DATA MODELS OF EFFORT ALLOCATION

* POOLED OLS:
regress  log_dt complex2 complex3 tau_d tau  logev cti cti2 cti3 obj_diff

* POOLED OLS WITH CLUSTERED STANDARD ERRORS:
regress  log_dt complex2 complex3 tau_d tau logev cti cti2 cti3 obj_diff ///
 , vce(cluster i)

* FIXED EFFECTS AND RANDOM EFFECTS; STORE POSTERIOR RANDOM EFFECT 
* HAUSMAN TEST TO DECIDE BETWEEN FE AND RE

xtreg log_dt complex2 complex3 tau_d tau logev cti cti2 cti3 obj_diff, fe
est store fe

xtreg log_dt complex2 complex3 tau_d tau logev cti cti2 cti3 obj_diff, re
est store re
predict u_hat, u
label variable u_hat "posterior random effect"

hausman fe re

* EXAMINE POSTERIOR RANDOM EFFECT:

summ u_hat if t==1
hist u_hat if t==1

* CHAPTER 6: DEALING WITH DISCRETENESS IN EXPERIMENTAL DATA

*6.2.2 MODELLING CHOICES BETWEEN LOTTERIES (THE HOUSE MONEY EFFECT)

use "house_money_sim", clear
set more off

* TABLE SHOWING PROPORTION OF SAFE CHOICES AT EACH WEALTH LEVEL:

table w, contents(n y mean y)

* THE PROBIT MODEL AND WALD TEST; STORE ESTIMATES FOR LATER USE:

probit y w
test w=0
est store with_w

* 6.2.3 MARGINAL EFFECTS

margins, dydx(w) at(w=0)
margins, dydx(w) at(w=10)
margins, dydx(w) 

* 6.2.4 LR TEST

probit y
est store without_w

lrtest with_w without_w

*6.2.5  ANALYSIS OF ULTIMATUM GAME DATA

use ug_sim, clear

* ANALYZE RESPONDER'S DECISION AND COMPUTE MIMIMUM ACCEPTABLE OFFER:

tab d
lowess d y
probit d y
nlcom MAO: -_b[_cons]/_b[y]

*6.2.6 THE STRATEGY METHOD

use ug_sm_sim, clear

ci MAO
tab d


* 6.3  THE ML ROUTINE IN STATA

capture program drop myprobit
set more off

* LOG-LIKELIHOOD EVALUATION PROGRAM "myprobit" STARTS HERE:

program define myprobit

* SPECIFY NAME OF QUANTITY WHOSE SUM WE WISH TO MAXIMISE (logl)
* AND ALSO PARAMETER NAMES (EMBODIED IN xb)
* PROVIDE LIST OF TEMPORARY VARIABLES (p ONLY)

	args logl xb
	tempvar p 

* GENERATE PROBABILITY OF CHOICE MADE BY EACH SUBJECT (p):	
	
	quietly gen double `p'=normal(yy*`xb')
	
* TAKE NATURAL LOG OF p AND STORE THIS AS logl
	
	quietly replace `logl'=ln(`p')
	
* END "myprobit" PROGRAM:	
	
	end

* READ DATA	

use "house_money_sim", clear

* GENERATE (INTEGER) yy FROM y:

gen int yy=2*y-1

* SPECIFY LIKELIHOOD EVALUATOR (lf), EVALUATION PROGRAM (myprobit), 
* AND EXPLANATORY VARIABLE LIST.
* RUN MAXIMUM LIKELIHOOD PROCEDURE
	
ml model lf myprobit ( = w)
ml maximize

*6.4  STRUCTURAL MODELLING

capt program drop structural
program structural
	args logl r sig
	tempvar eus eur diff p 

	quietly gen double `eus'=(w+5)^(1-`r')/(1-`r')
	quietly gen double `eur'=0.5*w^(1-`r')/(1-`r')+0.5*(w+10)^(1-`r')/(1-`r')
	quietly gen double `diff'=(`eus'-`eur')/`sig'
	quietly gen double `p'=normal(yy*`diff')
	quietly replace `logl'=ln(`p')
	end

ml model lf structural /r  /sig
ml maximize


*6.5.1 THE HETEROGENEOUS AGENT MODEL

use holtlaury_sim, clear

probit y rstar

nlcom (mu: -_b[_cons]/_b[rstar]) (sig: -1/_b[rstar])

mat V=e(V)
mat list V

* 6.6.1 INTERVAL DATA

use "interval_data_sim", clear

intreg rlower rupper

intreg rlower rupper age male

* 6.6.2  EXACT DATA

use "exact_sim", clear

hist r

program define exact
	args lnf xb sig
	tempvar y p

	quietly gen double `y'=$ML_y1
	quietly gen double `p'=(1/`sig')*normalden((`y'-`xb')/`sig')
	quietly replace `lnf'=ln(`p')
	end

ml model lf exact (r= ) () 
ml maximize

summ r

* 6.6.3  CENSORED DATA

use bardsley, clear

lowess y med, msize(0.4) jitter(1) bwidth(0.7) xlabel(0(1)10)
lowess y tsk, msize(0.4) jitter(1) bwidth(0.7) xlabel(0(5)30)

tobit y med tsk, ll(0) ul(10)

regress y med tsk


* 6.7.1 THE ULTIMATUM GAME: FURTHER ANALYSIS

use ug_sim, clear

gen m_to_f=male_i*(1-male_j)

regress y male_i male_j m_to_f

probit d y male_j

nlcom more_to_male: -_b[male_j]/_b[y]

gen egal=y==50
tab egal

tab male_i egal , chi2

* CHAPTER 7: ORDINAL DATA

* 7.5 APPLICATION TO DATA ON EMOTIONS

use emotions, clear

oprobit irritation take_rate female_resp
oprobit contempt take_rate female_resp
oprobit envy take_rate female_resp
oprobit anger take_rate female_resp

probit destroyyesno irritation contempt envy anger female_resp 

* APPLICATION TO DATA ON STRENGTH OF PREFERENCE

use house_money_sim, clear

probit y w

mat b_p=e(b)
mat V_p=e(V)

oprobit ys w

mat b_op=e(b)
mat V_op=e(V)

scalar  hausman=(b_p[1,1]-b_op[1,1])^2/(V_p[1,1]-V_op[1,1])

scalar list hausman


* CHAPTER 8: MIXTURE MODELS

* 8.2 MIXTURE OF TWO NORMALS

program drop _all
set more off

*  LIKELIHOOD EVALUATION PROGRAM STARTS HERE:

program define mixture
	args logl mu1 sig1 mu2 sig2 p
	tempvar f1 f2

* GENERATE TYPE-CONDITIONAL DENSITIES:	
	
	quietly gen double `f1'=(1/`sig1')*normalden((y-`mu1')/`sig1')
	quietly gen double `f2'=(1/`sig2')*normalden((y-`mu2')/`sig2')
	
* COMBINE TYPE-CONDITIONAL DENSITIES WITH MIXING PROPORTIONS 
* TO GENERATE MARGINAL DENSITY, AND TAKE LOG.  THIS IS THE FUNCTION THAT
* NEEDS TO BE MAXIMISED WHEN SUMMED OVER THE SAMPLE:
		
	quietly replace `logl'=ln(`p'*`f1'+(1-`p')*`f2')

* GENERATE THE POSTERIOR TYPE PROBABILITIES, AND MAKE THEM
* AVAILABLE OUTSIDE THE PROGRAM:
	
	quietly replace postp1=`p'*`f1'/(`p'*`f1'+(1-`p')*`f2')
	quietly replace postp2=(1-`p')*`f2'/(`p'*`f1'+(1-`p')*`f2')

	quietly putmata postp1, replace
	quietly putmata postp2, replace
	
	end

* END OF LIKELIHOOD EVALUATION PROGRAM	
	
* READ DATA:
	
use mixture_sim, clear	

* INITIALISE TWO POSTERIOR PROBABILITY VARIABLES:
	
gen postp1=.
gen postp2=.

* SPECIFY STARTING VALUES, AND APPLY ML:

mat start=(3, 1.5, 6, 1.5, .5)
ml model lf mixture /mu1 /sig1 /mu2 /sig2 /p
ml init start, copy
ml maximize

* EXTRACT POSTERIOR TYPE PROBABILITY, AND PLOT THEM AGAINST y:

drop postp1 postp2
getmata postp1
getmata postp2

sort y
line  postp1 postp2 y , lpattern(l -)

* 8.3 FMM command

fmm y, mix(normal) comp(2)

* 8.4 ACQUIRING A COMPANY TASK

use acquire_sim, clear	

program define acquire1
	args lnf mu1 sig1 mu2 sig2 p
	tempvar y f1 f2

	quietly gen double `y'=$ML_y1
	quietly gen double `f1'=(1/`sig1')*normalden((`y'-`mu1')/`sig1')
	quietly replace `f1'=normal(-`mu1'/`sig1') if y==0
	quietly replace `f1'=1-normal((100-`mu1')/`sig1') if y==100
	quietly gen double `f2'=(1/`sig2')*normalden((`y'-`mu2')/`sig2')
	quietly replace `f2'=normal(-`mu2'/`sig2') if y==0
	quietly replace `f2'=1-normal((100-`mu2')/`sig2') if y==100
	quietly replace `lnf'=ln(`p'*`f1'+(1-`p')*`f2')

	end

mat start=(0, 20, 60, 20, .8)
ml model lf acquire1 (y= ) () () () ()
ml init start, copy

ml maximize

* 8.5 A MIXTURE MODEL FOR GIVING IN A PUBLIC GOODS EXPERIMENT

prog drop _all
set more off

* LIKELIHOOD EVALUATION PROGRAM STARTS HERE:

program define pg_mixture
	args todo b logl
	tempvar p1_1 p2_1 p3_1 p1_2 p2_2 p3_2 p1_3 p2_3 p3_3 p1 p2 p3 pp1 pp2 pp3 pp w
		
	tempname xb1 xb2 sig1 sig2 w0 w1 p_rec p_str

* ASSIGN PARAMETER NAMES TO THE ELEMENTS OF THE PARAMETER VECTOR b:
	
	mleval `xb1' = `b', eq(1) 
	mleval `xb2' = `b', eq(2) 
	mleval `sig1' = `b', eq(3) scalar
	mleval `sig2'=`b', eq(4) scalar
	mleval `w0'=`b', eq(5) scalar
	mleval `w1'=`b', eq(6) scalar
	mleval `p_rec'=`b', eq(7) scalar
	mleval `p_str'=`b', eq(8) scalar
	
	quietly{
	
* INITIALISE THE p* VARIABLES WITH MISSING VALUES:
	
	gen double `p1_1'=.
	gen double `p2_1'=.
	gen double `p3_1'=.
	gen double `p1_2'=.
	gen double `p2_2'=.
	gen double `p3_2'=.
	gen double `p1_3'=.
	gen double `p2_3'=.
	gen double `p3_3'=.
	
	gen double `p1'=.
	gen double `p2'=.
	gen double `p3'=.
	
	gen double `pp1'=.
	gen double `pp2'=.
	gen double `pp3'=.
	gen double `pp'=.
	
* GENERATE THE TREMBLE PROBABILITY:
	
	gen double `w'=`w0'*exp(`w1'*tsk_1)

* COMPUTE TYPE-CONDITIONAL DENSITIES UNDER REGIME 1:
			
		replace `p1_1'=(1-`w')*normal(-`xb1'/`sig1')+`w'/11
		replace `p2_1'=(1-`w')*normal(-`xb2'/`sig2')+`w'/11
		replace `p3_1'=1-(10/11)*`w'

* COMPUTE TYPE-CONDITIONAL DENSITIES UNDER REGIME 2:
		
		replace `p1_2'=(1-`w')*(1/`sig1')*normalden((y-`xb1')/`sig1')+`w'/11
		replace `p2_2'=(1-`w')*(1/`sig2')*normalden((y-`xb2')/`sig2')+`w'/11
		replace `p3_2'=`w'/11

* COMPUTE TYPE-CONDITIONAL DENSITIES UNDER REGIME 3:
		
		replace `p1_3'=(1-`w')*(1-normal((10-`xb1')/`sig1'))+`w'/11
		replace `p2_3'=(1-`w')*(1-normal((10-`xb2')/`sig2'))+`w'/11
		replace `p3_3'=`w'/11

* MATCH TYPE-CONDITIONAL DENSITIES TO ACTUAL REGIMES (d IS REGIME):		
		
		replace `p1' = (d==1)*`p1_1'+(d==2)*`p1_2'+(d==3)*`p1_3'
		replace `p2' = (d==1)*`p2_1'+(d==2)*`p2_2'+(d==3)*`p2_3'
		replace `p3' = (d==1)*`p3_1'+(d==2)*`p3_2'+(d==3)*`p3_3'

* FIND PRODUCT OF TYPE-CONDITIONAL DENSITIES FOR EACH SUBJECT:		

		by i: replace `pp1'=exp(sum(ln(max(`p1',1e-12))))
		by i: replace `pp2'=exp(sum(ln(max(`p2',1e-12))))
		by i: replace `pp3'=exp(sum(ln(max(`p3',1e-12))))

* COMBINE TYPE-CONDITIONAL DENSITIES TO OBTAIN MARGINAL DENSITY FOR EACH SUBJECT
* (ONLY REQUIRED IN FINAL ROW FOR EACH SUBJECT):
		
		replace `pp'=`p_rec'*`pp1'+`p_str'*`pp2'+(1-`p_rec'-`p_str')*`pp3'
		replace `pp'=. if last~=1

* SPECIFY (LOG-LIKELIHOOD) FUNCTION WHOSE SUM OVER SUBJECTS IS TO BE MAXIMISED
		
		mlsum `logl'=ln(`pp') if last==1

* GENERATE POSTERIOR TYPE PROBABILITIES, AND MAKE THESE AVAILABLE OUTSIDE THE PROGRAM
		
		replace postp1=`p_rec'*`pp1'/`pp'
		replace postp2=`p_str'*`pp2'/`pp'
		replace postp3=(1-`p_rec'-`p_str')*`pp3'/`pp'
		
		putmata postp1, replace
		putmata postp2, replace
		putmata postp3, replace
	
	}
					
	end

* END OF LOG-LIKELIHOOD EVALUATION PROGRAM
	
clear
set more off

* READ DATA

use "bardsley", clear

by i: gen last=_n==_N

gen int d=1
replace d=2 if y>0
replace d=3 if y==10

gen double ord_1=ord-1
gen double tsk_1=tsk-1

* SET MEDIAN OF PREVIOUS CONTRIBUTIONS TO 8 FOR SUBJECTS IN FIRST POSITION:

replace med=8 if ord==1

*  SPECIFY EXPLANATORY-VARIABLE LISTS FOR RECIPROCATOR (LIST1) 
*  AND STRATEGIST (LIST2) EQUATIONS:

local list1 "med tsk_1"
local list2 "ord_1 tsk_1"

* INITIALISE VARIABLES TO BE USED FOR POSTERIOR TYPE PROBABILITIES:

gen postp1=.
gen postp2=.
gen postp3=.

* SPECIFY STARTING VALUES:

mat start=(0.57,-0.10,6.1,-0.93,-0.05,5.2,3.3,3.7,0.11,-0.05,0.26,0.49)

* SPECIFY LIKELIHOOD EVALUATOR, PROGRAM, AND PARAMETER NAMES:

ml model d0 pg_mixture (=`list1') (=`list2') /sig1 /sig2 /w0  /w1  /p_rec /p_str
ml init start, copy

* USE ML COMMAND TO MAXIMISE LOG-LIKELIHOOD, AND STORE RESULTS AS "WITH_TREMBLE":

ml max, trace search(norescale)
est store with_tremble

* COMPUTE THIRD MIXING PROPORTION USING DELTA METHOD:

nlcom p_fr: 1-[p_rec]_b[_cons]-[p_str]_b[_cons]

* EXTRACT POSTERIOR TYPE PROBABILITIES AND PLOT THEM AGAINST 
*  NUMBER OF ZERO CONTRIBUTIONS:

drop postp1 postp2 postp3

getmata postp1
getmata postp2
getmata postp3

label variable postp1 "rec"
label variable postp2 "str"
label variable postp3 "fr"

by i: gen n_zero=sum(y==0)

scatter postp1 postp2 postp3 n_zero if last==1, title("with tremble") ///
ytitle("posterior probability") msymbol(x Dh Sh) jitter(3) saving(with, replace)

* ESTIMATE MODEL WITHOUT TREMBLE, AND STORE RESULTS AS "WITHOUT_TREMBLE":

constraint 1 [w0]_b[_cons]=0.00
constraint 2 [w1]_b[_cons]=0.00

ml model d0 pg_mixture (=`list1') (=`list2') ///
/sig1 /sig2 /w0  /w1  /p_rec /p_str, constraints(1 2)

ml init start, copy
ml max, trace search(norescale)
est store without_tremble

nlcom p_fr: 1-[p_rec]_b[_cons]-[p_str]_b[_cons]

* EXTRACT AND PLOT POSTERIOR TYPE PROBABILITIES FOR MODEL WITHOUT TREMBLE:

drop postp1 postp2 postp3

getmata postp1
getmata postp2
getmata postp3

label variable postp1 "rec"
label variable postp2 "str"
label variable postp3 "fr"

scatter postp1 postp2 postp3 n_zero if last==1, title("without tremble") ///
ytitle("posterior probability") msymbol(x Dh Sh) jitter(3) saving(without, replace)

* CARRY OUT LIKELIHOOD RATIO TEST FOR PRESENCE OF TREMBLE:

lrtest with_tremble without_tremble

* COMBINE THE TWO POSTERIOR PROBABILITY PLOTS

gr combine with.gph without.gph

erase with.gph
erase without.gph

* CHAPTER 9: SIMULATING EXPERIMENTAL DATA

* 9.2 RANDOM NUMBER GENERATION IN STATA

clear

set obs 1000
set seed 7654321

gen double z=rnormal()
gen double x=3+2*z
hist x, normal

* 9.3.1 SIMULATING DATA FROM A LINEAR MODEL

clear

set obs 100
set seed 7654321

gen double x=runiform()
gen double u=rnormal()
gen double y=2.0+1.0*x+0.5*u
scatter y x, ylabel(0(1)4)
regress y x

* MONTE CARLO SIMULATION OF LINEAR MODEL

capt prog drop ols_monte
   
   program define ols_monte, rclass
        syntax [, obs(integer 1)  ]
        set obs `obs'
		keep x
		tempvar u y 

		gen double u=rnormal()

		gen y=2.0+1.0*x+0.5*u

		regress y x

		return scalar slope = _b[x]
		return scalar intercept = _b[_cons]
		return scalar rmse=e(rmse)
 
        end

	clear
	set obs 100
	gen double x=runiform()
    simulate slope=r(slope) intercept=r(intercept) rmse=r(rmse), reps(1000): ols_monte, obs(100)

summ
	
* 9.3.2 SIMULATING PANEL DATA 

clear

set obs 1000

egen int i=seq(), f(1) t(50) b(20)
egen int t=seq(), f(1) t(20)
xtset i t

set seed 7654321

gen x=runiform()
gen e=rnormal()

by i: generate double u=0.5*rnormal() if _n==1
by i: replace u=u[1] if u==.

gen double y=2+0.5*x+u+e

xtreg y x, re

* 9.3.3 SIMULATING DYNAMIC PANEL DATA

clear

set obs 1000

egen int i=seq(), f(1) t(50) b(20)
egen int t=seq(), f(1) t(20)
xtset i t

set seed 654321

gen x=runiform()
gen e=rnormal()

by i: generate double u=0.5*rnormal() if _n==1
by i: replace u=u[1] if u==.

by i: gen double y=0 if _n==1
by i: replace y=0.0+0.3*y[_n-1]+0.5*x+u+e  if y==.

xtreg y x, re

xtabond y x

* 9.3.3 SIMULATING BINARY PANEL DATA (MONTE CARLO)

capt prog drop panel_monte
   
   program define panel_monte, rclass
        syntax [, obs(integer 1)  ]
        set obs `obs'
		keep x
		tempvar i t z1 z2 ee e u v ys y
	

egen i=seq(), f(1) t(50) b(20)
egen t=seq(), f(1) t(20)
tsset i t


gen double e=rnormal()

by i: generate u=rnormal() if _n==1
by i: replace u=u[1] if u==.

gen ys=-1.0+2.0*x+u+e
gen y=ys>0

probit y x
return scalar slope_pooled = _b[x]

xtprobit y x, re i(i)
return scalar slope_re = _b[x]

end


clear
set obs 1000
gen double x=runiform()
simulate slope_pooled=r(slope_pooled) slope_re=r(slope_re), reps(1000): panel_monte, obs(1000)


hist slope_re, xline(2) xlabel(1(0.2)3)
hist slope_pooled, xline(2) xlabel(1(0.2)3)

* 9.4 MONTE CARLO INVESTIGATION OF THE HAUSMAN TEST

prog drop _all

   program define hausman, rclass
        syntax [, obs(integer 1)  ]
        drop _all
        set obs `obs'

* GENERATE SUBJECT ID AND WEALTH (w)		
		
egen subj=fill(1/2)
egen w=seq(), f(0) t(20) b(50)
replace w=w/2

* GENERATE LATENT VARIABLE y*

gen ystar=1.5-0.2*w+rnormal()

* GENERATE BINARY VARIABLE y_bin

gen y_bin=ystar>0

* SET CUT-POINTS

scalar k1=-1
scalar k2=-0.5
scalar k3=0
scalar k4=.5
scalar k5=1

* GENERATE TRUTHFUL OUTCOME "y_good"

gen y_good=1+(ystar>k1)+(ystar>k2)+(ystar>k3)+(ystar>k4)+(ystar>k5)

* GENERATE "COMPLETELY RANDOM" OUTCOME "y_bad"

gen u=uniform()
gen three=1+(u>0.33)+(u>0.67)
gen y_bad=3*y_bin+three

* GENERATE PARTIALLY RANDOM OUTCOME "y_poor"

gen y_poor=y_good
replace y_poor=y_bad if uniform()<0.3

* ESTIMATE BINARY PROBIT

probit y_bin w
mat b_p=e(b)
mat V_p=e(V)

* ESTIMATE ORDERED PROBIT USING TRUTHFUL OUTCOME (I.E. UNDER NULL)

oprobit y_good w
mat b_op=e(b)
mat V_op=e(V)

* COMPUTE HAUSMAN TEST STATISTIC UNDER NULL

return scalar  hausman1=(b_p[1,1]-b_op[1,1])^2/(V_p[1,1]-V_op[1,1])

* ESTIMATE ORDERED PROBIT USING PARTIALLY RANDOM OUTCOME (I.E. UNDER ALTERNATIVE)

oprobit y_poor w
mat b_op=e(b)
mat V_op=e(V)

* COMPUTE HAUSMAN TEST STATISTIC UNDER ALTERNATIVE

return scalar hausman2=(b_p[1,1]-b_op[1,1])^2/(V_p[1,1]-V_op[1,1])

end

* RUN SIMULATION, WITH 1000 REPLICATIONS; SET SAMPLE SIZE TO 1050

simulate hausman1=r(hausman1) hausman2=r(hausman2) , reps(1000): hausman, obs(1050)

* INVESTIGATE TEST SIZE

summ hausman1, detail
qchi hausman1, df(1) msize(1)

* INVESTIGATE TEST POWER

hist hausman2, xline(3.84 6.64)

* CHAPTER 10: INTRODUCTION TO THE METHOD OF MAXIMUM SIMULATED LIKELIHOOD (MSL)

* 10.3.2 GENERATING HALTON DRAWS IN STATA

* NOTE: YOU NEED TO INSTALL THE FUNCTION "MDRAWS"

* GENERATE 2 HALTONS AND 2 RANDOM UNIFORMS, AND COMPARE PLOTS PLOT

clear
set obs 1000
matrix p=(2,3)
mdraws, neq(2) dr(1) prefix(h) primes(p)
gen u1=uniform()
gen u2=uniform()

scatter u2 u1, saving(normal, replace) 
scatter  h2_1 h1_1, saving(halton, replace) 

graph combine normal.gph halton.gph

erase normal.gph
erase halton.gph

* CONVERT TO NORMALS AND SHOW HISTOGRAMS

gen z_random=invnorm(u1)
gen z_halton=invnorm(h1_1)

hist z_random, normal ylabel(0(0.1)0.5) saving(normal, replace)
hist z_halton, normal  ylabel(0(0.1)0.5) saving(halton, replace)

graph combine normal.gph halton.gph

erase normal.gph
erase halton.gph

* 10.3.3 HALTON DRAWS FOR PANEL DATA ESTIMATION

clear

* SET SAMPLE SIZE

set obs 30

*  GENERATE SUBJECT IDENTIFIER (i) AND TASK IDENTIFIER (t);
* ENSURE THAT THESE ARE STORED AS INTEGERS

egen i=seq(), f(1) t(10) b(3)
egen t=seq(), f(1) t(3)

recast int i t

*  DECLARE DATA TO BE PANEL DATA

xtset i t

* GENERATE INDICATORS FOR THE "FIRST" AND "LAST" OBSERVATION WITHIN EACH BLOCK

by i: generate int first=1 if _n==1
by i: generate int last=1 if _n==_N

* GENERATE HALTONS IN FIRST ROW OF EACH BLOCK

mat p=[3]
mdraws if first==1, neq(1) dr(3) prefix(h) primes(p)
scalar draws=r(n_draws)

* COPY INTO OTHER ROWS TO CREATE PANEL STRUCTURE

local hlist h1*

quietly{

foreach v of varlist `hlist' {
recast double `v'
by i: replace `v'=`v'[1] if `v'==.
replace `v'=invnorm(`v')
}
}

*NOW INSPECT DATA IN DATA BROWSER

* 10.4 RANDOM EFFECTS PROBIT

clear
set more off

* SET SAMPLE SIZE AND RANDOM NUMBER SEED

set obs 1200
set seed 7654321

*  GENERATE SUBJECT IDENTIFIER (i) AND TASK IDENTIFIER (t);
* ENSURE THAT THESE ARE STORED AS INTEGERS
egen i=seq(), f(1) b(20)
egen t=seq(), f(1) t(20)
recast int i t

*  DECLARE DATA TO BE PANEL DATA

xtset i t

* GENERATE x (FROM UNIFORM) AND e (FROM NORMAL) 

gen double x=runiform()
gen double e=rnormal()

* GENERATE u (SUBJECT-SPECIFIC EFFECT)

by i: generate double u=0.5*(rnormal()) if _n==1
by i: replace u=u[1] if u==.

* GENERATE LATENT VARIABLE y*, AND BINARY VARIABLES y and yy

gen double ystar=-1.0+2.0*x+u+e
gen int y=ystar>0
gen int yy=2*y-1

* ESTIMATE RANDOM EFFECTS PROBIT MODEL USING xtprobit COMMAND

xtprobit y x

* GENERATE INDICATOR VARIABLES FOR FIRST AND LAST OBSERVATION FOR EACH SUBJECT

by i: gen int first=1 if _n==1
by i: gen int last=1 if _n==_N

* APPEND (HORIZONTALLY) EACH SUBJECT'S FIRST ROW WITH 125 HALTON DRAWS 
* (DIFFERENT BETWEEN SUBJECTS).  STORE NUMBER OF DRAWS AS "draws".

mat p=[3]
mdraws if first==1, neq(1) dr(125) prefix(h) primes(p)
scalar draws=r(n_draws)

*CREATE A VARIABLE LIST CONTAINING THE HALTON DRAWS
* ENSURE THEY ARE IN DOUBLE PRECISION
* COPY THE ROW OF HALTONS IN EACH BLOCK INTO ROWS 2-T OF SAME BLOCK

local hlist h1*

quietly{
foreach v of varlist `hlist' {
recast double `v'
by i: replace `v'=`v'[1] if `v'==.
replace `v'=invnorm(`v')
}
}

* LIKELIHOOD EVALUATION PROGRAM "my_rep" STARTS HERE:

capt prog drop my_rep
program define my_rep

* SPECIFY ARGUMENTS

	args todo b lnppp
	tempvar xb p pp ppp upp uppp
	tempname ln_s_u s_u
	local hlist h1*

* EXTRACT ELEMENTS OF PARAMETER VECTOR b	
	
	mleval `xb' = `b', eq(1)
	mleval  `ln_s_u' = `b', eq(2) scalar
	scalar `s_u'=exp(`ln_s_u')

* INITIALISE TEMPORARY VARIABLES	
	
	quietly gen double `p'=.
	quietly gen double `pp'=.
	quietly gen double `ppp'=0
	quietly gen double `upp'=.
	quietly gen double `uppp'=0

* LOOP FOR EVALUATION OF SUM (OVER r) OF PRODUCT (OVER t)
* pp AND ppp ARE FOR LIKELIHOOD FUNCTION;
* upp AND uppp ARE FOR NUMERATOR OF POSTERIOR RANDOM EFFECT FORMULA	
	
	quietly{
		foreach v of varlist `hlist' {
		replace  `p'= normal(yy*(`xb' + `s_u'*`v'))
		by i: replace `pp' = exp(sum(ln(`p')))
		replace `pp'=. if last~=1
		replace `upp'=`s_u'*`v'*`pp'
		replace `ppp'=`ppp'+`pp'
		replace `uppp'=`uppp'+`upp'
		}
		
* DIVISION BY R TO GENERATE REQUIRED AVERAGES (OVER r)
* COMPUTE POSTERIOR RANDOM EFFECT VARIABLE (u_post) AND SEND THIS TO MATA 
		
		quietly {
		replace `ppp'=`ppp'/draws
		replace `uppp'=`uppp'/draws
		replace u_post=`uppp'/`ppp'
		}
		putmata u_post, replace
		
* MLSUM COMMAND TO SPECIFY PER-SUBJECT LOG-LIKELIHOOD CONTRIBUTION		
		
		mlsum `lnppp'=ln(`ppp') if last==1
		}
end

* "end" SIGNIFIES END OF LIKELIHOOD EVALUATION PROGRAM "my_rep"

* CREATE VARIABLE LIST (list_explan) FOR EXPLANATORY VARIABLES;
* ESTIMATE SIMPLE PROBIT MODEL
* STORE ESTIMATES FROM SIMPLE PROBIT MODEL
* CREATE VECTOR OF STARTING VALUES (start) FOR PANEL PROBIT MODEL
* INITIALISE VARIABLE CONTAINING POSTERIOR RANDOM EFFECT (u_post) 

local list_explan "x"
probit y `list_explan'
mat b_probit=e(b)
mat start = b_probit,0
gen double u_post=.

* SPECIFY EVALUATOR (d0), EVALUATION PROGRAM (my_rep), AND PARAMETER LIST
* SPECIFY STARTING VALUE VECTOR
* RUN MAXIMUM LIKELIHOOD PROCEDURE; DEDUCE ESTIMATE OF s_u USING DELTA METHOD

ml model d0 my_rep (  = `list_explan')  /ln_s_u
ml init start, copy
ml max
nlcom s_u: exp(_b[ln_s_u:_cons])

* EXTRACT POSTERIOR RANDOM EFFECT (u_post) GENERATED INSIDE EVALUATION PROGRAM
* PLOT IT AGAINST TRUE RANDOM EFFECT (u).

drop u_post
getmata u_post
lowess u_post u, xline(0) yline(0)


* 10.5 THE RANDOM EFFECTS TWO-LIMIT TOBIT MODEL

* LIKELIHOOD EVALUATION PROGRAM "my_ret" STARTS HERE:

capt prog drop my_ret
program define my_ret

* SPECIFY ARGUMENTS

	args todo b logl
	tempvar xb p pp ppp upp uppp
	tempname ln_s_u s_u ln_s_e s_e
	local hlist h1*

* EXTRACT ELEMENTS OF PARAMETER VECTOR b	
	
	mleval `xb' = `b', eq(1)
	mleval  `ln_s_u' = `b', eq(2) scalar
	mleval `ln_s_e' = `b', eq(3) scalar
	scalar `s_u'=exp(`ln_s_u')
	scalar `s_e'=exp(`ln_s_e')
	
* INITIALISE TEMPORARY VARIABLES	
	
	quietly gen double `p'=.
	quietly gen double `pp'=.
	quietly gen double `ppp'=0
	quietly gen double `upp'=.
	quietly gen double `uppp'=0

* LOOP FOR EVALUATION OF SUM (OVER r) OF PRODUCT (OVER t)
* pp AND ppp ARE FOR LIKELIHOOD FUNCTION;
* upp AND uppp ARE FOR NUMERATOR OF POSTERIOR RANDOM EFFECT FORMULA	
	
	quietly{
		foreach v of varlist `hlist' {
		replace  `p'= normal(-(`xb' + `s_u'*`v')/`s_e') if y==0
		replace  `p'=(1/`s_e')*normalden((y-`xb'-`s_u'*`v')/`s_e') if (y>0)&(y<10)
		replace  `p'= normal((-10+(`xb' + `s_u'*`v'))/`s_e') if y==10
		by i: replace `pp' = exp(sum(ln(`p')))
		replace `pp'=. if last~=1
		replace `upp'=`s_u'*`v'*`pp'
		replace `ppp'=`ppp'+`pp'
		replace `uppp'=`uppp'+`upp'
		}
		
* DIVISION BY R TO GENERATE REQUIRED AVERAGES (OVER r)
* COMPUTE POSTERIOR RANDOM EFFECT VARIABLE (u_post) AND SEND THIS TO MATA 
		
		quietly {
		replace `ppp'=`ppp'/draws
		replace `uppp'=`uppp'/draws
		replace u_post=`uppp'/`ppp'
		}
		putmata u_post, replace
	
* MLSUM COMMAND TO SPECIFY PER-SUBJECT LOG-LIKELIHOOD CONTRIBUTION		
		
		mlsum `logl'=ln(`ppp') if last==1
		}
end

* "END" SIGNIFIES END OF LIKELIHOOD EVALUATION PROGRAM "my_rep" 

* READ DATA AND DECLARE TO BE PANEL DATA

use bardsley, clear
xtset i t

* GENERATE INDICATOR VARIABLES FOR FIRST AND LAST OBSERVATION FOR EACH SUBJECT

by i: gen int first=1 if _n==1
by i: gen int last=1 if _n==_N

* APPEND (HORIZONTALLY) EACH SUBJECT'S FIRST ROW WITH 125 HALTON DRAWS 
* (DIFFERENT BETWEEN SUBJECTS).  STORE NUMBER OF DRAWS AS "draws".

mat p=[3]
mdraws if first==1, neq(1) dr(125) prefix(h) primes(p)
scalar draws=r(n_draws)

*CREATE A VARIABLE LIST CONTAINING THE HALTON DRAWS
* ENSURE THEY ARE IN DOUBLE PRECISION
* COPY THE ROW OF HALTONS IN EACH BLOCK INTO ROWS 2-T OF SAME BLOCK

local hlist h1*

quietly{
foreach v of varlist `hlist' {
recast double `v'
by i: replace `v'=`v'[1] if `v'==.
replace `v'=invnorm(`v')
}
}

* CREATE VARIABLE LIST (list_explan) FOR EXPLANATORY VARIABLES;
* ESTIMATE 2-LIMIT TOBIT MODEL
* STORE ESTIMATES FROM 2-LIMIT TOBIT MODEL
* CREATE VECTOR OF STARTING VALUES (start) FOR RANDOM EFFECTS 2-LIMIT TOBIT MODEL
* INITIALISE VARIABLE CONTAINING POSTERIOR RANDOM EFFECT (u_post) 

local list_explan "med tsk"
tobit y `list_explan', ll(0) ul(10)
mat b_tobit=e(b)
mat ln_s_e=ln(b_tobit[1,4])
mat start=b_tobit[1,1..3],0,ln_s_e
gen double u_post=.

* SPECIFY EVALUATOR (d0), EVALUATION PROGRAM (my_rep), AND PARAMETER LIST
* SPECIFY STARTING VALUE VECTOR
* RUN MAXIMUM LIKELIHOOD PROCEDURE; DEDUCE ESTIMATES OF s_u and s_e USING DELTA METHOD

ml model d0 my_ret (  = `list_explan')  /ln_s_u  /ln_s_e
ml init start, copy
ml max
nlcom (s_u: exp(_b[ln_s_u:_cons])) (s_e: exp(_b[ln_s_e:_cons])) 

* EXTRACT POSTERIOR RANDOM EFFECT (u_post) GENERATED INSIDE EVALUATION PROGRAM
* PLOT POSTERIOR RANDOM EFFECT AGAINST SUBEJECT'S MEAN CONTRIBUTION

drop u_post
getmata u_post
by i: egen mean_y=mean(y)
scatter u_post mean_y, yline(0)


*  CHAPTER 11: DEALING WITH ZEROS: HURDLE MODELS

* 11.5: THE PANEL HURDLE MODEL (SIMULATION)

clear
set more off

* SET RANDOM NUMBER SEED

set seed 971156

* SET SAMPLE SIZE; GENERATE SUBJECT NUMBER (i) AND TASK NUMBER (t)

set obs 2000

egen int i=seq(), f(1)  b(20)
egen int t=seq(), f(1) t(20)

summ i
scalar N=r(max)
summ t
scalar T=r(max)

xtset i t

* SET PARAMETER VALUES

scalar a1=2.0
scalar a0=-1.5
scalar b1=10.0
scalar b0=-5.0
scalar sig_u=2.5
scalar sig_e=5.0
scalar rho=0.40
scalar w=0.04
scalar y_max=10

* GENERATE VARIABLES

gen double x=uniform()
gen double e2=invnorm(uniform())

by i: generate double u=(invnorm(uniform())) if _n==1
by i: replace u=u[1] if u==.

by i: generate double z=uniform() if _n==1
by i: generate double e1=rho*u+sqrt(1-rho^2)*invnorm(uniform()) if _n==1

generate double ds=a0+a1*z+e1
generate int d=ds>0

by i: replace z=z[1]
by i: replace d=d[1]

gen double yss=b0+b1*x+sig_u*u+sig_e*e2
gen double ys=yss*(yss>0)
gen double y=ys*d

replace y=y_max if y>y_max

gen int tremble=uniform()<w
replace y=y_max*uniform() if tremble==1

* DROP ALL LATENT VARIABLES

drop e1 e2 u ds d yss ys tremble

* GENERATE INDICATORS FOR FIRST AND LAST OBSERVATION FOR EACH SUBJECT

by i: generate int first=1 if _n==1
by i: generate int last=1 if _n==_N

* GENERATE HALTON DRAWS IN FIRST ROW OF EACH SUBJECT
* STORE NUMBER OF DRAWS AS "DRAWS"

mat p=[3]
mdraws if first==1 , neq(1) dr(31) prefix(h) primes(p) burn(3)
scalar draws=r(n_draws)

* COLLECT HALTON DRAWS IN VARIABLE LIST "hlist"

local hlist h1*
recast double h1*

* FOR EACH SUBJECT, COPY ROW OF HALTON DRAWS INTO EVERY ROW; 
*  CONVERT TO STANDARD NORMAL

quietly{

foreach v of varlist `hlist' {
by i: replace `v'=`v'[1] if `v'==.
replace `v'=invnorm(`v')
}
}

* LIKELIHOOD EVALUATION PROGRAM ("PANEL_HURDLE") STARTS HERE:

program drop _all
program define panel_hurdle

* SPECIFY ARGUMENTS

	args todo b logl
	tempvar za xb pp0 p1 pp1 ppp1 pp ppp pd ppd 
	tempname s_u s_e rr w
	local hlist h1*

* EXTRACT ELEMENTS OF PARAMETER VECTOR b		
		
	mleval `za' = `b', eq(1) 
	mleval `xb' = `b', eq(2)
	mleval `s_u' = `b', eq(3) scalar
	mleval `s_e' = `b', eq(4) scalar
	mleval `rr' = `b', eq(5) scalar
	mleval `w'=`b', eq(6) scalar
	
* INITIALISE TEMPORARY VARIABLES
		
	quietly gen double `p1'=.
	quietly gen double `pp1'=.
	quietly gen double `ppp1'=0
	quietly gen double `pp0'=.
	quietly gen double `pp'=.
	quietly gen double `ppp'=0
	quietly gen double `pd'=.
	quietly gen double `ppd'=0
		
* LOOP FOR EVALUATION OF SUM (OVER r) OF PRODUCT (OVER t)
* ppp IS FOR LIKELIHOOD FUNCTION; ppd IS FOR p(d=1) (USED TO COMPUTE 
* POSTERIOR PROBABILITY OF d=0)
		
	quietly{
		
		foreach v of varlist `hlist' {
		replace  `p1'= (1-`w')*(1/`s_e')*normalden((y-(`xb' + `s_u'*`v')) ///
				/`s_e')+`w'/y_max if (y>0)&(y<y_max)
		replace  `p1'=(1-`w')*(1-normal((`xb' + `s_u'*`v')/`s_e')) if y==0
		replace `p1'=(1-`w')*(1-normal((y_max-(`xb' + `s_u'*`v'))/`s_e')) ///
				if y==y_max
		by i: replace `pp1' = exp(sum(ln(`p1')))
		replace `pp1'=. if last~=1
		replace `ppp1'=`ppp1'+`pp1'
				
		replace `pd'=normal((`za'+`rr'*`v')/sqrt(1-`rr'^2))
		replace `pd'=. if last~=1
				
		replace `pp0'= ((`w'/y_max)^n_pos)*((1-`w')^n_zero)
		replace `pp0'=. if last~=1
				
		replace `pp'=`pd'*`pp1'+(1-`pd')*`pp0'
		replace `ppp'=`ppp'+`pp'
		
		replace `ppd'=`ppd'+`pd'
		}	
* END OF LOOP
		
* DIVISION BY R TO GENERATE REQUIRED AVERAGES (OVER r)
* COMPUTE POSTERIOR PROBABILITY OF d=0 (pd0) AND SEND THIS TO MATA 
		
		replace `ppp'=`ppp'/draws
		
		replace `ppp1'=`ppp1'/draws
		replace `ppd'=`ppd'/draws
				
		replace pd0=`pp0'*(1-`ppd')/(`pp0'*(1-`ppd')+`ppp1'*`ppd')
		putmata pd0, replace
		
* MLSUM COMMAND TO SPECIFY PER-SUBJECT LOG-LIKELIHOOD CONTRIBUTION	
		
		mlsum `logl'=ln(`ppp') if last==1
						
		}
end

* "end" SIGNIFIES END OF LIKELIHOOD EVALUATION PROGRAM "panel_hurdle"

* GENERATE BINARY d INDICATING AT LEAST ONE POSITIVE CONTRIBUTION BY SUBJECT; 
* GENERATE NUMBER OF POSITIVE CONTRIBUTIONS BY SUBJECT (n_pos) AND NUMBER OF 
* ZERO CONTRIBUTIONS (n_zero)

quietly{

by i: gen int sum_y=sum(y) 
by i: replace sum_y=sum_y[_N]

gen int d=sum_y>0

gen int y_pos=y>0

by i: gen int n_pos = sum(y_pos)
by i: replace n_pos = n_pos[_N]

gen int y_zero=y==0
by i: gen int n_zero = sum(y_zero)
by i: replace n_zero = n_zero[_N]

}


* INITIALISE VARIABLE TO REPRESENT POSTERIOR PROBABILITY OF
* FALLING AT FIRST HURDLE (pd0)

quietly gen double pd0=.

* USE PROBIT (1 OBS PER SUBJECT) TO OBTAIN FIRST HURDLE STARTING VALUES

probit d x   if last==1
mat bprobit=e(b)

* USE RANDOM EFFECTS TOBIT TO OBTAIN SECOND HURDLE STARTING VALUES

xttobit y  x if d==1 , ll(0) ul(10)
mat bxttobit=e(b)

* DEFINE VECTOR OF STARTING VALUES (INCLUDING GUESSES FOR rho AND w)

mat start=bprobit, bxttobit, 0.00, 0.02

* SPECIFY EVALUATOR (d0), EVALUATION PROGRAM (panel_hurdle), AND PARAMETER LIST
* SPECIFY STARTING VALUE VECTOR
* RUN MAXIMUM LIKELIHOOD PROCEDURE

ml model d0 panel_hurdle ( = z ) (  = x ) /s_u /s_e /rr /w
ml init start, copy 
ml max, trace search(norescale)

* EXTRACT POSTERIOR PROBABILITY OF d=0 (pd0) GENERATED INSIDE EVALUATION PROGRAM
* PLOT IT AGAINST NUMBER OF POSITIVE CONTRIBUTIONS.

drop pd0
getmata pd0

label variable pd0 "posterior prob zero type"
label variable n_pos "number of positive contributions"

scatter pd0 n_pos, jitter(1)  ylabel(0(0.1)1)



* 11.6 A PANEL HURDLE MODEL OF DICTATOR GAME GIVING

clear
clear matrix
clear mata

set more off

use erkal, clear

sort i t
 
recast int   i t male  treatment myrank1 myrank2 yourrank1 yourrank2
recast double send_ij age

recast double pos_diff_words neg_diff_words

rename pos_diff_words pos_wd
rename neg_diff_words neg_wd

rename send_ij y
rename treatment treat

gen double age2=age^2


tab treat, gen(treat)
tab myrank2, gen(myrank2)
tab yourrank2, gen(yourrank2)

gen double pd0=.
label variable pd0 "posterior probability selfish"
gen double pp0=.
gen double pp1=.
gen double pd=.

xtset i t

by i: generate int first=1 if _n==1
by i: generate int last=1 if _n==_N

by i: generate double sum_y=sum(y) 
replace sum_y=. if last~=1

egen double max_y=max(y)

mat p=[3]
mdraws if first==1, neq(1) dr(32) prefix(h) primes(p)

scalar draws=r(n_draws)

recast double h1*

local hlist h1*

quietly{

foreach v of varlist `hlist' {
by i: replace `v'=`v'[1] if `v'==.
replace `v'=invnorm(`v')
}
}

*end of loop

program drop _all
program define panel_hurdle
	version 11
	args todo b lnf
	tempvar za xb z p0 pp0 ppp0 p1 pp1 ppp1 pp ppp pd ppd y_t
	tempname ln_s_u ln_s_e s_u s_e rr rrr ll y_t_min w
	local hlist h1*
		
	mleval `za'= `b', eq(1)	
	mleval `xb' = `b', eq(2) 
	mleval `s_u' = `b', eq(3) scalar
	mleval `s_e' = `b', eq(4) scalar
	mleval `rr' = `b', eq(5) scalar
	mleval `w' = `b', eq(6) scalar
	
		
	quietly gen double `p1'=.
	quietly gen double `pp1'=.
	quietly gen double `ppp1'=0
		
	quietly gen double `p0'=.
	quietly gen double `pp0'=.
	quietly gen double `ppp0'=0	
		
	quietly gen double `pp'=.
	quietly gen double `ppp'=0
	
	quietly gen double `pd'=.
	quietly gen double `ppd'=0
	
	*scalar `rr'=tanh(`rrr')

		
		
	quietly{
		
		foreach v of varlist `hlist' {
		replace  `p1'= (1-`w')*(1/`s_e')*normalden((y-(`xb' + `s_u'*`v'))/`s_e')+`w'/20 if y>0
		replace  `p1'=(1-`w')*(1-normal((`xb'+ `s_u'*`v')/`s_e'))+`w'/2 if y==0
		by i: replace `pp1' = exp(sum(ln(`p1')))
		replace `pp1'=. if last~=1
				
		replace `pd'=normal((`za'+`rr'*`v')/sqrt(1-`rr'^2))
		replace `pd'=. if last~=1
		
		replace `pp0'= ((`w'/20)^n_pos)*((1-`w'/2)^n_zero)
		replace `pp0'=. if last~=1
		
		replace `pp'=`pd'*`pp1'+(1-`pd')*`pp0'
		replace `ppp'=`ppp'+`pp'
				
		}	
		*end of loop
		
		replace `ppp'=`ppp'/draws
				
		mlsum `lnf'=ln(`ppp') if last==1
		
		
		replace pd0=`pp0'*(1-`pd')/(`pp0'*(1-`pd')+`pp1'*`pd')
		replace pp0=`pp0'
		replace pp1=`pp1'
		replace pd=`pd'
		
		putmata pd0, replace
		putmata pp0, replace
		putmata pp1, replace
		putmata pd, replace
		}
end


capt drop d
gen double d=sum_y>0


gen int y_pos=.
gen int n_pos=.
label variable n_pos "number of positive transfers"
gen int y_zero=.
gen int n_zero=.



quietly{

by i: replace sum_y=sum(y) 
by i: replace sum_y=sum_y[_N]

replace   y_pos=y>0
by i: replace  n_pos = sum(y_pos)
by i: replace n_pos = n_pos[_N]

replace y_zero=y==0
by i: replace  n_zero = sum(y_zero)
by i: replace n_zero = n_zero[_N]


replace d=sum_y>0

}


* CREATE VARIABLE LISTS

local listd "male  myrank22 myrank23 myrank24  "
local listy "male   yourrank22 yourrank23 yourrank24   "

* USE ONLY TREATMENTS 1 AND 2

keep if treat<3

* OBTAIN STARTING VALUES USING PROBIT AND XTTOBIT.  
* STORE VARIANCE ESTIMATES FROM XTTOBIT

probit d   `listd'      if last==1
mat bprobit=e(b)

xttobit y  `listy'  if  d==1, ll(0)   
mat bxttobit=e(b)

scalar sig_u=_b[sigma_u:_cons]
scalar sig_e=_b[sigma_e:_cons]

* CREATE STARTING VALUE VECTOR

mat start=bprobit, bxttobit,0,0

* SPECIFY CONSTRAINTS ON PARAMETERS
   
constraint 3 [sig_u]_b[_cons]=sig_u  
constraint 4 [sig_e]_b[_cons]=sig_e      
constraint 5 [r]_b[_cons]=0.0
constraint 6 [w]_b[_cons]=0  

* ESTIMATE PANEL HURDLE MODEL WITH sig_u, sig_e, r, w ALL CONSTRAINED

ml model d0 panel_hurdle (= `listd') ( = `listy' ) /sig_u /sig_e  /r /w, ///
 constraints( 3 4 5 6)
ml init start, copy
ml max, trace  search(norescale)

* STORE ESTIMATES AS STARTING VECTOR FOR NEXT ESTIMATION

mat start=e(b)

* ESTIMATE PANEL HURDLE MODEL WITH r AND w CONSTRAINED

ml model d0 panel_hurdle (= `listd' ) ( = `listy') /sig_u /sig_e  /r /w, ///
 constraints( 5 6)
ml init start, copy
ml max, trace  search(norescale)

* STORE ESTIMATES AS STARTING VECTOR FOR NEXT ESTIMATION

mat start=e(b)

* ESTIMATE PANEL HURDLE MODEL WITH ONLY w CONSTRAINED (SET STARTING VALUE FOR r)

mat start[1,`=colnumb(start, "r:_cons")']=-0.3

ml model d0 panel_hurdle (= `listd' ) ( = `listy') /sig_u /sig_e  /r /w, ///
  constraints(  6)
ml init start, copy
ml max, trace  search(norescale)


drop pd0
drop pp0
drop pp1
drop pd
		
getmata pd0
getmata pp0
getmata pp1
getmata pd

label variable pd0 "posterior probability zero type"
label variable n_pos "number of positive transfers"

scatter pd0 n_pos, msize(1) jitter(1)

* MANN-WHITNEY TEST OF EFFECT OF GENDER ON CONTRIBUTION

ranksum y, by(male)


* 11.7 A PANEL HURDLE MODEL OF CONTRIBUTION IN A PUBLIC GOODS GAME


clear

set more off

use "clark.dta", clear

gen house=treatment==1

gen tt2=treatment==2
gen tt3=treatment==3
gen tt4=treatment==4

gen tt24=tt2+tt4
gen tt24t=tt24*t
gen tt2t=tt2*t

xtset i t

gen t2=t^2
gen t3=t^3

keep if treatment<3

*TRY OUT DIFFERENT ASSUMPTIONS ABOUT PREVIOUS AT START


gen double meanoth = (15*mean_y - y)/14

replace previous = L.meanoth 


replace previous=66 if previous==.
gen double previous2=previous^2


by i: generate int first=1 if _n==1
by i: generate int last=1 if _n==_N

by i: generate double sum_y=sum(y) 
replace sum_y=. if last~=1

egen double max_y=max(y)

mat p=[3]
mdraws if first==1, neq(1) dr(32) prefix(h) primes(p)

scalar draws=r(n_draws)

recast double h1*

local hlist h1*

quietly{

foreach v of varlist `hlist' {
by i: replace `v'=`v'[1] if `v'==.
replace `v'=invnorm(`v')
}
}

*end of loop

quietly gen double pd0=.
quietly gen double pp0=.
quietly gen double ppp1=.
quietly gen double pd=.


program drop _all
program define my_rep
	version 11
	args todo b lnf
	tempvar az bx z p0 pp0 ppp0 p1 pp1 ppp1 pp ppp pd ppd
	tempname ln_s_u ln_s_e s_u s_e rr rrr w
	local hlist h1*
		
	mleval `az' = `b', eq(1) 
	mleval `bx' = `b', eq(2)
	mleval `s_u' = `b', eq(3) scalar
	mleval `s_e' = `b', eq(4) scalar
	mleval `rr' = `b', eq(5) scalar
	mleval `w'=`b', eq(6) scalar
	
		
	quietly gen double `p1'=.
	quietly gen double `pp1'=.
	quietly gen double `ppp1'=0
		
	quietly gen double `p0'=.
	quietly gen double `pp0'=.
	quietly gen double `ppp0'=0	
		
	quietly gen double `pp'=.
	quietly gen double `ppp'=0
	
	quietly gen double `pd'=.
	quietly gen double `ppd'=0
	
		
	
	quietly{
		
		foreach v of varlist `hlist' {
		replace  `p1'= (1-`w')*(1/`s_e')*normalden(($ML_y2-(`bx' + `s_u'*`v'))/`s_e')+`w'/(2*max_y) if $ML_y2>0
		replace  `p1'=(1-`w')*(1-normal((`bx' + `s_u'*`v')/`s_e'))+`w'/2 if $ML_y2==0
		replace `p1'=(1-`w')*(1-normal((80-(`bx' + `s_u'*`v'))/`s_e'))+`w'/max_y if $ML_y2==80
		by i: replace `pp1' = exp(sum(ln(`p1')))
		replace `pp1'=. if last~=1
		replace `ppp1'=`ppp1'+`pp1'
				
		replace `pd'=normal((`az'+`rr'*`v')/sqrt(1-`rr'^2))
		replace `pd'=. if last~=1
		
		
		replace `pp0'= ((`w'/(2*max_y))^n_pos)*((1-`w'/2)^n_zero)
		replace `pp0'=. if last~=1
				
			
		
		replace `pp'=`pd'*`pp1'+(1-`pd')*`pp0'
		replace `ppp'=`ppp'+`pp'
		
		
		
		}	
		*end of loop
		
		replace `ppp'=`ppp'/draws
		mlsum `lnf'=ln(`ppp') if last==1
		
		replace `ppp1'=`ppp1'/draws
		
		replace `pd'=normal((`az')/sqrt(1-`rr'^2))
		replace pd0=`pp0'*(1-`pd')/(`pp0'*(1-`pd')+`ppp1'*`pd')
		replace pp0=`pp0'
		replace ppp1=`ppp1'
		replace pd=`pd'
		
		putmata pd0, replace
		putmata pp0, replace
		putmata ppp1, replace
		putmata pd, replace
		
		}
end


capt drop d
gen d=sum_y>0


gen int y_pos=.
gen int n_pos=.
gen int y_zero=.
gen int n_zero=.



quietly{

by i: replace sum_y=sum(y) 
by i: replace sum_y=sum_y[_N]

replace   y_pos=y>0
by i: replace  n_pos = sum(y_pos)
by i: replace n_pos = n_pos[_N]

replace y_zero=y==0
by i: replace  n_zero = sum(y_zero)
by i: replace n_zero = n_zero[_N]


replace d=sum_y>0

}




probit d house   if last==1

mat bprobit=e(b)

xttobit y   t  previous previous2 if d==1 , ll(0) ul(80)

mat bxttobit=e(b)

mat start=bprobit, bxttobit, 0.9, 0.08


constraint 5 [rr]_b[_cons]=0.00  
constraint 6 [w]_b[_cons]=0.00  

ml model d0 my_rep (d=  house ) (y  =     t  previous previous2  )  /s_u /s_e /rr /w, constraints(5 6)
ml init start, copy
ml max, trace search(norescale)

mat start=e(b)

ml model d0 my_rep (d=  house ) (y  =     t  previous previous2  )  /s_u /s_e /rr /w, constraints( 6)
ml init start, copy
ml max, trace search(norescale)



drop pd0
drop pp0
drop ppp1
drop pd
getmata pd0
getmata pp0
getmata ppp1
getmata pd

label variable pd0 "posterior prob free rider"
label variable n_pos "number of positive contributions"

scatter pd0 n_pos, jitter(1) ylabel(0(0.1)1) saving(without, replace)


mat start=e(b)
mat start[1,6]=0.08

ml model d0 my_rep (d=  house ) (y  =     t  previous previous2  )  /s_u /s_e /rr /w, constraints( )
ml init start, copy
ml max, trace search(norescale)


drop pd0
drop pp0
drop ppp1
drop pd
getmata pd0
getmata pp0
getmata ppp1
getmata pd

gen hhouse="O"
replace hhouse="H" if house==1

label variable pd0 "posterior prob free rider"
label variable n_pos "number of positive contributions"

scatter pd0 n_pos, jitter(1) mlabel(hhouse) ylabel(0(0.1)1)  saving(with, replace)

graph combine without.gph with.gph

* CHAPTER 13: CHOICE UNDER RISK: ECONOMETRIC MODELLING

* 13.3 SIMULATION AND ESTIMATION

prog drop _all
set more off

* LIKELIHOOD EVALUATION PROGRAM FOR FECHNER MODEL STARTS HERE:

program define fechner

* DECLARE VARIABLES, PARAMETERS, AND VARIABLE LIST FOR HALTON DRAWS

	args todo b logl
	tempvar pp1 pp2 pp3 qq1 qq2 qq3 dd1 dd2 dd3 r w z p pp ppp rpp rppp
	tempname mu ln_eta eta sig w0 w1 aa gg
	local hlist h1*

* EXTRACT SCALAR PARAMETERS FROM VECTOR b.	
	
	mleval `mu' = `b', eq(1) scalar
	mleval `eta' = `b', eq(2) scalar
	mleval `sig' = `b', eq(3) scalar
	mleval `w0'=`b', eq(4) scalar
	mleval `w1'=`b', eq(5) scalar
	mleval `aa'=`b', eq(6) scalar
	mleval `gg'=`b', eq(7) scalar

* INITIALISE VARIABLES	
	
	quietly{
	gen double `pp1'=.
	gen double `pp2'=.
	gen double `pp3'=.
	gen double `qq1'=.
	gen double `qq2'=.
	gen double `qq3'=.
	gen double `dd1'=.
	gen double `dd2'=.
	gen double `dd3'=.
	gen double `r'=.
	gen double `w'=`w0'*exp(`w1'*tau)
	gen double `z'=.
	gen double `p'=.
	gen double `pp'=.
	gen double `rpp'=.
	gen double `ppp'=0
	gen double `rppp'=0
	}

	quietly{
	
* TRANSFORM TRUE PROBABILITIES (p1,p2,p3,q1,q2,q3) USING PRELEC WEIGHTING FUNCTION.
* TRANSFORMED PROBABILITIES ARE pp1,pp2,pp3,qq1,qq2,qq3:
	
replace `pp3'=exp(-`aa'*(-ln(p3))^`gg')
replace `pp3'=0 if p3==0
replace `pp2'=exp(-`aa'*(-ln(p3+p2))^`gg')-`pp3'
replace `pp2'=0 if p2==0
replace `pp1'=1-`pp2'-`pp3'

replace `qq3'=exp(-`aa'*(-ln(q3))^`gg')
replace `qq3'=0 if q3==0
replace `qq2'=exp(-`aa'*(-ln(q3+q2))^`gg')-`qq3'
replace `qq2'=0 if q2==0
replace `qq1'=1-`qq2'-`qq3'

* GENERATE DIFFERENCE VARIABLES FROM TRANSFORMED PROBABILITIES:

replace `dd3'=`pp3'-`qq3'
replace `dd2'=`pp2'-`qq2'
replace `dd1'=`pp1'-`qq1'

* START LOOP OVER HALTON VARIABLES:

		foreach v of varlist `hlist' {
		
* GENERATE LIKELIHOOD FOR EACH ROW:		
		
		replace `r'= exp(`mu'+ `eta'*`v')
		replace `z'=(`dd1'*(x1^(`r'))+`dd2'*(x2^(`r'))+`dd3'*(x3^(`r')))/`sig'
		replace  `p'= (1-`w')*normal(yy*`z')+`w'/2
		
* TAKE PRODUCT WITHIN EACH SUBJECT, AND PLACE THIS IN LAST ROW FOR EACH SUBJECT:		
		
		by i: replace `pp' = exp(sum(ln(max(`p',1e-12))))
		replace `pp'=. if last~=1
		
* ADD COLUMN OF PRODUCTS TO THE CUMULATIVE SUM BUILDING UP BETWEEN LOOPS:		
		
		replace `ppp'=`ppp'+`pp'

* USE SIMILAR PROCEDURE TO CREATE VARIABLE FOR USE IN COMPUTATION OF 
* POSTERIOR MEAN OF RISK ATTITUDE		
		
		replace `rpp'=`r'*`pp'
		replace `rpp'=. if last~=1
		replace `rppp'=`rppp'+`rpp'
		}

* END OF LOOP

* FIND MEANS OF VARIABLES GENERATED BY LOOP:
		
		replace `ppp'=`ppp'/draws
		replace `rppp'=`rppp'/draws
		
		replace lppp=ln(`ppp') if last==1
		replace rhat=`rppp'/`ppp'
		by i: replace rhat=rhat[_N] if rhat==.
		
* GENERATE SUBJECT-CONTRIBUTIONS TO LOG-LIKELIHOOD, AND DECLARE AS MAXIMAND	
		
		mlsum `logl'=ln(`ppp') if last==1

* USE PUTMATA TO MAKE SOME GLOBAL VARIABLES AVAILABLE OUTSIDE THE PROGRAM
* THE ddd_hat'S ARE NEEDED TO OBTAIN CLOSENESS TO INDIFFERENCE
* lppp IS NEEDED TO PERFORM NON-NESTED TESTS
* rhat IS POSTERIOR RISK ATTITUDE		
		
replace dd1_hat=`dd1'
replace dd2_hat=`dd2'
replace dd3_hat=`dd3'
		
putmata lppp, replace
putmata rhat, replace	
		
putmata dd1_hat, replace
putmata dd2_hat, replace
putmata dd3_hat, replace
	
	}
					
	end

* END OF FECHNER LIKELIHOOD EVALUATION PROGRAM

* LIKELIHOOD EVALUATION PROGRAM FOR RP STARTS HERE:

program define rp

* DECLARE VARIABLES, PARAMETERS, AND VARIABLE LIST FOR HALTON DRAWS

	args todo b logl
	tempvar pp1 pp2 pp3 qq1 qq2 qq3 dd1 dd2 dd3 r w astar z p pp ppp rpp rppp 
	tempname mu eta sig  w0 w1 aa gg
	local hlist h1*

* EXTRACT SCALAR PARAMETERS FROM VECTOR b	
	
	mleval `mu' = `b', eq(1) scalar
	mleval `eta' = `b', eq(2) scalar
	mleval `sig' = `b', eq(3) scalar
	mleval `w0'=`b', eq(4) scalar
	mleval `w1'=`b', eq(5) scalar
	mleval `aa'=`b', eq(6) scalar
	mleval `gg'=`b', eq(7) scalar
	
* INITIALISE VARIABLES	
	
	quietly{
	gen double `pp1'=.
	gen double `pp2'=.
	gen double `pp3'=.
	gen double `qq1'=.
	gen double `qq2'=.
	gen double `qq3'=.
	gen double `dd1'=.
	gen double `dd2'=.
	gen double `dd3'=.
	gen double `w'=`w0'*exp(`w1'*tau)
	gen double `r'=.
	gen double `z'=.
	gen double `p'=.
	gen double `pp'=.
	gen double `rpp'=.
	gen double `ppp'=0
	gen double `rppp'=0
	gen double `astar'=0

* TRANSFORM TRUE PROBABILITIES (p1,p2,p3,q1,q2,q3) USING PRELEC WEIGHTING FUNCTION.
* TRANSFORMED PROBABILITIES ARE pp1,pp2,pp3,qq1,qq2,qq3:	
	
replace `pp3'=exp(-`aa'*(-ln(p3))^`gg')
replace `pp3'=0 if p3==0
replace `pp2'=exp(-`aa'*(-ln(p3+p2))^`gg')-`pp3'
replace `pp2'=0 if p2==0
replace `pp1'=1-`pp2'-`pp3'

replace `qq3'=exp(-`aa'*(-ln(q3))^`gg')
replace `qq3'=0 if q3==0
replace `qq2'=exp(-`aa'*(-ln(q3+q2))^`gg')-`qq3'
replace `qq2'=0 if q2==0
replace `qq1'=1-`qq2'-`qq3'

* GENERATE DIFFERENCE VARIABLES FROM TRANSFORMED PROBABILITIES:

replace `dd3'=`pp3'-`qq3'
replace `dd2'=`pp2'-`qq2'
replace `dd3'=1 if dom==1
replace `dd2'=-2 if dom==1
replace `dd1'=`pp1'-`qq1'
			
* START LOOP OVER HALTON VARIABLES:
			
		foreach v of varlist `hlist' {
		
* GENERATE LIKELIHOOD FOR EACH ROW:				
		
		replace `r'= `mu'+ `eta'*`v'
		replace `astar'=ln(-`dd2'/`dd3')/ln(2)
		replace `z'=(`r'-ln(`astar'))/`sig'
		replace  `p'= (1-`w')*((1-dom)*normal(yy*`z')+dom*y)+`w'/2

* TAKE PRODUCT WITHIN EACH SUBJECT, AND PLACE THIS IN LAST ROW FOR EACH SUBJECT:
				
		by i: replace `pp' = exp(sum(ln(max(`p',1e-12))))
		replace `pp'=. if last~=1
		
* ADD COLUMN OF PRODUCTS TO THE CUMULATIVE SUM BUILDING UP BETWEEN LOOPS:			
		
		replace `ppp'=`ppp'+`pp'
		
* USE SIMILAR PROCEDURE TO CREATE VARIABLE FOR USE IN COMPUTATION OF 
* POSTERIOR MEAN OF RISK ATTITUDE				
		
		replace `rpp'=`r'*`pp'
		replace `rpp'=. if last~=1
		replace `rppp'=`rppp'+`rpp'
		}

* END OF LOOP

* FIND MEANS OF VARIABLES GENERATED BY LOOP:
		
		replace `ppp'=`ppp'/draws
		replace `rppp'=`rppp'/draws
		replace lppp=ln(`ppp') if last==1
		
* GENERATE SUBJECT-CONTRIBUTIONS TO LOG-LIKELIHOOD, AND DECLARE AS MAXIMAND	
		
		mlsum `logl'=ln(`ppp') if last==1

* USE PUTMATA TO MAKE GLOBAL VARIABLE lppp AVAILABLE OUTSIDE THE PROGRAM
* lppp IS NEEDED TO PERFORM NON-NESTED TESTS
		
putmata lppp, replace	
	}
					
	end

* END OF RP LIKELIHOOD EVALUATION PROGRAM
	
* SIMULATION STARTS HERE

clear
set more off

* READ DATA SET CONTAINING SUBJECT NUMBER (i) TASK NUMBER (t), AND
* PROBABALITIES DEFINING LOTTERIES (p1-p3;q1-q3)

use "design.dta", clear

* SET RANDOM NUMBER SEED; RECAST VARIABLES TO CORRECT TYPE

set seed 91611143
recast int i t
recast double p* q*

* GENERATE SCALARS N AND T

summ i
scalar N=r(max)
summ t
scalar T=r(max)

* SET TRUE VALUES OF PARAMETERS FOR SIMULATION

scalar mu=-0.88
scalar eta=0.2
scalar sig_fechner=0.05
scalar sig_rp=0.15
scalar w0=0.06
scalar w1=-0.01
scalar aa=0.9 
scalar gg=0.8

* SIMULATE POSITION OF PROBLEM IN SEQUENCE (tau)
* REMEMBER THAT 50 PROBLEMS ARE SET IN TWO SITTINGS

by i: gen  int d50=_n>50
bysort i d50: egen tau_d=rank(uniform())
gen int tau=50*d50+tau_d

* SIMULATE RISK ATTITUDE FOR FECHNER MODEL;
* REMEMBER THAT THIS NEEDS TO BE FIXED WITHIN A SUBJECT

by i: generate double r_fechner=exp(mu+eta*(invnorm(uniform()))) if _n==1
by i: replace r_fechner=r_fechner[1] if r_fechner==.

* SIMULATE RISK ATTITUDE FOR RP MODEL;
* FIRST SIMULATE SUBJECT MEAN m FROM LOGNORMAL;
* THEN SIMULATE RISK ATTITUDE AS LOGNORMAL WITH MEAN PARAMETER m

by i: generate double  m=mu+eta*(invnorm(uniform())) if _n==1
by i: replace m=m[1] if m==.
gen double r_rp=exp(m+sig_rp*invnorm(uniform()))

* GENERATE TRANSFORMED PROBABILITIES FROM TRUE PROBABILITIES USING
* PRELEC WEIGHTING FUNCTION; TRANSFORMED PROBABILITIES ARE (pp1-pp3; qq1-qq3)

gen double pp3=exp(-aa*(-ln(p3))^gg)
replace pp3=0 if p3==0
gen double pp2=exp(-aa*(-ln(p3+p2))^gg)-pp3
replace pp2=0 if p2==0
gen double pp1=1-pp2-pp3

gen double qq3=exp(-aa*(-ln(q3))^gg)
replace qq3=0 if q3==0
gen double qq2=exp(-aa*(-ln(q3+q2))^gg)-qq3
replace qq2=0 if q2==0
gen double qq1=1-qq2-qq3

* GENERATE 3 (CONSTANT) VARIABLES REPRESENTING THE THREE MONEY AMOUNTS

gen double x1=0
gen double x2=1
gen double x3=2

* COMPUTE VALUATION OF THE TWO LOTTERIES UNDER FECHNER:

gen double vp_fechner=pp1*(x1^r_fechner)+pp2*(x2^r_fechner)+pp3*(x3^r_fechner)
gen double vq_fechner=qq1*(x1^r_fechner)+qq2*(x2^r_fechner)+qq3*(x3^r_fechner)

* COMPUTE VALUATION OF THE TWO LOTTERIES UNDER RP:

gen double  vp_rp=pp1*(x1^r_rp)+pp2*(x2^r_rp)+pp3*(x3^r_rp)
gen double vq_rp=qq1*(x1^r_rp)+qq2*(x2^r_rp)+qq3*(x3^r_rp)

* GENERATE TREMBLE PROBABILITY (AS DECAYING FUNCTION OF tau)
* THEN SIMULATE TREMBLE INDICATOR (trem)

gen double w=w0*exp(w1*tau)
gen int trem=(uniform()<w)

* SIMULATE CHOICE FROM FECHNER MODEL; EXPRESS AS BOTH y AND yy:

gen int y_fechner=(1-trem)*((vp_fechner-vq_fechner ///
+sig_fechner*invnorm(uniform()))>0)+trem*(uniform()>0.5)
gen int yy_fechner=2*y_fechner-1

* SIMULATE CHOICE FROM RP MODEL; EXPRESS AS BOTH y AND yy:

gen int y_rp=(1-trem)*((vp_rp-vq_rp)>0)+trem*(uniform()>0.5)
gen int yy_rp=2*y_rp-1

* END OF SIMULATION

* ESTIMATION STARTS HERE:

* GENERATE INDICATOR FOR DOMINANCE PROBLEMS (dom):

gen double d1=p1-q1
gen double d2=p2-q2
gen double d3=p3-q3
gen int dom = (d3>=0)*((d3+d2)>=0)

* GENERATE INDICTATORS FOR FIRST AND LAST OBSERVATIONS FOR EACH SUBJECT:

by i: generate int  first=1 if _n==1
by i: generate int last=1 if _n==_N

* GENERATE HALTON DRAWS (31 COLUMNS) USING mdraws; CREATE VARIABLE LIST

mat p=[3]
mdraws if first==1 , neq(1) dr(31) prefix(h) primes(p) burn(3)
scalar draws=r(n_draws)
local hlist h1*

* ENSURE THAT HALTON DRAWS ARE IN DOUBLE PRECISION

recast double h1*

* IMPOSE PANEL STRUCTURE ON HALTON DRAWS: T ROWS ALL SAME WITHIN SUBJECT:

quietly{
foreach v of varlist `hlist' {
by i: replace `v'=`v'[1] if `v'==.
replace `v'=invnorm(`v')
}
}

* INITALISE VARIABLES REPRESENTING ESTIMATED DIFFERENCES OF 
* TRANSFORMED PROBABILITIES

gen double dd1_hat=.
gen double dd2_hat=.
gen double dd3_hat=.

* INITIALISE VARIABLES REPRESENTING: PER-SUBJECT LOG-LIKELIHOOD; 
* POSTERIOR RISK ATTITUDE

gen double lppp=.
gen double rhat=.

* INITIALISE CHOICE VARIABLES (y AND yy)

gen int y=.
gen int yy=.

* ASSIGN FECHNER CHOICE DATA TO y AND yy

replace yy=yy_fechner
replace y=y_fechner

* SET STARTING VALUES OF FECHNER MODEL:

mat start=(-0.68, 0.12, 0.10, 0.04, -0.005, 1.0, 1.0)

* ESTIMATE FECHNER MODEL ON FECHNER DATA USING ML

ml model d0 fechner /mu /eta  /sig /w0  /w1  /aa /gg
ml init start, copy
ml max

* EXTRACT GLOBAL VARIABLES FROM INSIDE FECHNER PROGRAM

drop lppp rhat dd1_hat dd2_hat dd3_hat
getmata lppp
getmata rhat
getmata dd1_hat dd2_hat dd3_hat 

* RENAME VARIABLE CONTAINING PER-SUBJECT LOG-LIKELIHOOD

rename lppp lp_fechner

* GENERATE VALUATION DIFFERENTIAL

gen double diff=dd1_hat*(x1^rhat)+dd2_hat*(x2^rhat)+dd3_hat*(x3^rhat)

* RE-INITIALISE VARIABLE CONTAINING PER-SUBJECT LOG-LIKELIHOOD  

gen lppp=. 

* ESTIMATE RP MODEL ON FECHNER DATA USING ML

ml model d0 rp /mu /eta  /sig /w0  /w1  /aa /gg
ml init start, copy
ml max

* EXTRACT VARIABLE CONTAINING PER-SUBJECT LOG-LIKELIHOOD FROM INSIDE RP PROGRAM

drop lppp 
getmata lppp
rename lppp lp_rp

* CARRY OUT VUONG'S NON-NESTED TEST (FOR MODELS ESTIMATED WITH FECHNER DATA):

gen vuong1=  lp_fechner - lp_rp
summ vuong1
scalar vuong=(sqrt(r(N))*r(mean))/(r(sd))
scalar list vuong

* CARRY OUT CLARKE'S NONPARAMETRIC NON-NESTED TESTS
* (FOR MODELS ESTIMATED WITH FECHNER DATA):

signrank lp_fechner = lp_rp
signtest lp_fechner = lp_rp

* NOW SWITCH TO RP DATA

* RE-INITIALISE VARIABLE CONTAINING PER-SUBJECT LOG-LIKELIHOOD  

gen double lppp=.

* ASSIGN RP CHOICE DATA TO y AND yy

replace yy=yy_rp
replace y=y_rp

* ESTIMATE FECHNER MODEL ON RP DATA USING ML

ml model d0 fechner /mu /eta  /sig /w0  /w1  /aa /gg
ml init start, copy
ml max

* EXTRACT VARIABLE CONTAINING PER-SUBJECT LOG-LIKELIHOOD 
* FROM INSIDE FECHNER PROGRAM

drop lppp lp_fechner
getmata lppp
rename lppp lp_fechner

* RE-INITIALISE VARIABLE CONTAINING PER-SUBJECT LOG-LIKELIHOOD  

gen double lppp=.

* ESTIMATE RP MODEL ON RP DATA USING ML

ml model d0 rp /mu /eta  /sig /w0  /w1  /aa /gg
ml init start, copy
ml max

* EXTRACT VARIABLE CONTAINING PER-SUBJECT LOG-LIKELIHOOD FROM INSIDE RP PROGRAM

drop lppp lp_rp
getmata lppp
rename lppp lp_rp

* CARRY OUT VUONG'S NON-NESTED TEST (FOR MODELS ESTIMATED WITH RP DATA):

replace vuong1=  lp_fechner - lp_rp
summ vuong1
scalar vuong=(sqrt(r(N))*r(mean))/(r(sd))
scalar list vuong

* CARRY OUT CLARKE'S NONPARAMETRIC NON-NESTED TESTS
* (FOR MODELS ESTIMATED WITH RP DATA):

signrank lp_rp = lp_fechner 
signtest lp_rp = lp_fechner 

* FINALLY, SIMULATE DECISION TIMES
* EXPLANATORY VARIABLES FOR DECISION TIME NEED TO BE GENERATED

* GENERATE COMPLEXITY LEVELS OF CHOICE PROBLEMS; 
* VARIABLE "complex" IS NUMBER OF OUTCOMES IN SIMPLER LOTTERY

gen complex_p=(p1>0)+(p2>0)+(p3>0)
gen complex_q=(q1>0)+(q2>0)+(q3>0)
gen complex=min(complex_p, complex_q)
gen complex2=complex==2
gen complex3=complex==3

* GENERATE LOG OF EXPECTED VALUE OF SAFE LOTTERY

gen logev=ln(q1*0+q2*1+q3*2)

* GENERATE ABSOLUTE VALUATION DIFFERENTIAL, AND ITS SQUARE AND CUBE

gen abs_diff=abs(diff)
gen abs_diff2=abs_diff^2
gen abs_diff3=abs_diff^3

* GENERATE MEASURE OF OBJECTIVE DIFFERENCE BETWEEN LOTTERIES

gen obj_diff=d1^2+d2^2+d3^2

* SIMULATE SUBJECT-SPECIFIC RANDOM EFFECT (dt1) FOR DECISION TIME:

by i: generate double  dt1=1.624+0.386*(invnorm(uniform())) if _n==1
by i: replace dt1=dt1[1] if dt1==.

* SIMULATE LOG OF DECISION TIME:

gen dt2=dt1+0.267*complex2+0.388*complex3-0.0019*(tau_d-1) ///
-0.0032*(tau-1)+0.028*logev-5.251*abs_diff+10.944*abs_diff2 ///
-7.339*abs_diff3+0.157*obj_diff+0.616*invnorm(uniform())

* GENERATE DECISION TIME:

gen dt=exp(dt2)


* CHAPTER 15: SOCIAL PREFERENCE MODELS

* 15.2.2 THE ANDREONI-MILLER DATA

use garp, clear
set more off

scatter x2 x1, msize(0.3) jitter(1)

lowess p2x2 m

lowess x2 p2, saving(self, replace)
lowess x2 p1, saving(other, replace)

graph combine self.gph other.gph

erase self.gph 
erase other.gph

regress x2 p2 p1, vce(cluster i)
regress x2 p2 p1 m, vce(cluster i)

tobit x2 p2 p1 m, vce(cluster i) ll(0)

xtset i t

xttobit x2 p2 p1 m, ll(0)

* 15.2.3 ESTIMATING THE PARAMETERS OF A CES UTILITY FUNCTION

nl (w1 = (p1^({rho}/({rho}-1)))/((p1^({rho}/({rho}-1)))  ///
+(({aa}/(1-{aa}))^(1/({rho}-1)))*(p2^({rho}/({rho}-1))))), ///
initial(rho 0.0 aa 0.5) vce(cluster i)

* USE DELTA METHOD TO ESTIMATE ELASTICITY OF SUBSTITUTION

nlcom sigma: 1/(1- _b[rho:_cons])

* 15.3 A MODEL OF ALTRUISM WITH BINDING NON-NEGATIVITY CONSTRAINTS

clear
clear matrix
clear mata

program drop _all
set more off

* LIKELIHOOD EVALUATION PROGRAM STARTS HERE

program define sg
	
	args todo b logl
	tempvar  gamma y d0 d_int dm w z p0 p_int pm f ff fff gff gfff
	tempname a1 mu s_u s_e 
	
	local hlist h1*
		
	mleval `a1'=`b', eq(1) scalar
	mleval `mu' = `b', eq(2) scalar
	mleval `s_u' = `b', eq(3) scalar
	mleval `s_e' = `b', eq(4) scalar
		
	quietly gen double `d0'=.
	quietly gen double `d_int'=.
	quietly gen double `dm'=.
	quietly gen double `gamma'=.
	
	quietly gen double `w'=.
	quietly gen double `z'=.
	
	quietly gen double `p0'=.
	quietly gen double `p_int'=.
	quietly gen double `pm'=.
	
	quietly gen double `f'=.
	quietly gen double `ff'=.
	quietly gen double `gff'=.
	quietly gen double `fff'=0
	quietly gen double `gfff'=0
				
	quietly{
		
		replace `d0'=x1<=0
		replace `d_int'=((p1*x1)>0)&((p1*x1)<m)
		replace `dm'=p1*x1>=m
		
		foreach v of varlist `hlist' {
		replace `gamma'= `mu'+ `s_u'*`v'
		
		replace `w'=(p1*x1-`a1'*m)/((1-`a1')*p1+`a1'*p2)
		replace `z'=(`w'-`gamma')/(`s_e')
		replace `p0'=normal(`z')
		replace `p_int'=(1/`s_e')*normalden(`z')
		replace `pm'=1-normal(`z')
		
		replace  `f'= `d0'*`p0'+`d_int'*`p_int'+`dm'*`pm'
		by i: replace `ff' = exp(sum(ln(max(`f',0.000000001))))
		replace `gff'=`gamma'*`ff'
		replace `ff'=. if last~=1
		replace `gff'=. if last~=1
		replace `fff'=`fff'+`ff'
		replace `gfff'=`gfff'+`gff'
				
		}
		
		replace `fff'=`fff'/draws
		replace `gfff'=`gfff'/draws
	
		mlsum `logl'=ln(`fff') if last==1
		}

	quietly  replace g_post=`gfff'/`fff'
		
	quietly{
	putmata g_post, replace
	}
					
	end

* END OF LIKELIHOOD EVALUATION PROGRAM

*READ DATA

use garp, clear

*INITIALISE POSTERIOR MEAN OF b1 

gen double g_post=.

drop if x1==.

bysort i: generate first=1 if _n==1
bysort i: generate last=1 if _n==_N

* GENERATE HALTON SEQUENCES

mat p=[3]
mdraws if first==1, neq(1) dr(32) prefix(h) primes(p)

scalar draws=r(n_draws)

local hlist h1*

quietly {
	foreach v of varlist `hlist'{
		by i: replace `v'=`v'[1] if `v'==.
		replace `v'=invnorm(`v')
		}
		}

* SET STARTING VALUES
		
mat start=(.7,5,10,10)

* RUN ML 

ml model d0 sg /a1 /mu /s_u /s_e
ml init start, copy
ml max, difficult

* EXTRACT POSTERIOR MEAN OF b1

drop g_post
getmata g_post

label variable g_post "posterior mean of self-subsistence-level"

hist g_post, bin(30) freq xlabel(-40(20)100)



* 15.4 FINITE MIXTURE MODELS OF ALTRUISM

*PROGRAM WITHOUT DEALING WITH UPPER CENSORING

prog drop _all
program define fairness1
	args lnf d sig p1 p2
	tempvar f1 f2 f3

	quietly gen double `f1'=(1/`sig')*normalden((y-`d'-me)/`sig')
	quietly gen double `f2'=(1/`sig')*normalden((y-`d'-ml)/`sig')
	quietly gen double `f3'=(1/`sig')*normalden((y-`d'-mle)/`sig')

	quietly replace `lnf'=ln(`p1'*`f1'+`p2'*`f2'+(1-`p1'-`p2')*`f3')
	
	quietly replace postp1=(`p1'*`f1')/(`p1'*`f1'+`p2'*`f2'+(1-`p1'-`p2')*`f3')
	quietly replace postp2=(`p2'*`f2')/(`p1'*`f1'+`p2'*`f2'+(1-`p1'-`p2')*`f3')
	quietly replace postp3=((1-`p1'-`p2')*`f3')/(`p1'*`f1'+`p2'*`f2'+(1-`p1'-`p2')*`f3')
	
	quietly putmata postp1, replace
	quietly putmata postp2, replace
	quietly putmata postp3, replace
	
	end

*PROGRAM DEALING WITH UPPER CENSORING

program define fairness2
	args lnf d sig p1 p2
	tempvar y f1 f2 f3

	quietly gen double `f1'=(1/`sig')*normalden((y-`d'-me)/`sig') if y<x
	quietly replace  `f1'=normal((`d'+me-x)/`sig') if y==x
	quietly gen double `f2'=(1/`sig')*normalden((y-`d'-ml)/`sig') if y<x
	quietly replace  `f2'=normal((`d'+ml-x)/`sig') if y==x
	quietly gen double `f3'=(1/`sig')*normalden((y-`d'-mle)/`sig') if y<x
	quietly replace  `f3'=normal((`d'+mle-x)/`sig') if y==x

	quietly replace `lnf'=ln(`p1'*`f1'+`p2'*`f2'+(1-`p1'-`p2')*`f3')
	
	quietly replace postp1=(`p1'*`f1')/(`p1'*`f1'+`p2'*`f2'+(1-`p1'-`p2')*`f3')
	quietly replace postp2=(`p2'*`f2')/(`p1'*`f1'+`p2'*`f2'+(1-`p1'-`p2')*`f3')
	quietly replace postp3=((1-`p1'-`p2')*`f3')/(`p1'*`f1'+`p2'*`f2'+(1-`p1'-`p2')*`f3')
	
	quietly putmata postp1, replace
	quietly putmata postp2, replace
	quietly putmata postp3, replace
		
	end


* READ DATA

use fairness_sim, clear

*GENERATE VARIABLES REPRESENTING FAIRNESS IDEALS (me, ml, mle)

gen me=x/2
gen ml=a1q1
gen mle=(q1/(q1+q2))*x

* INITIALISE VARIABLES REPRESENTING POSTERIOR TYPE PROBABILITIES

gen postp1=.
gen postp2=.
gen postp3=.	
	
* SET STARTING VALUES	
 
mat start=(20,10,.5,.2)

* ESTIMATE MODEL WITHOUT DEALING WITH CENSORING

ml model lf fairness1 /d /sig /p1 /p2
ml init start, copy
ml maximize
nlcom p3: 1- _b[p1:_cons]- _b[p2:_cons]

* ESTIMATE MODEL DEALING WITH CENSORING

ml model lf fairness2 /d /sig /p1 /p2
ml init start, copy
ml maximize
nlcom p3: 1- _b[p1:_cons]- _b[p2:_cons]

* EXTRACT AND PLOT POSTERIOR TYPE PROBABILITIES

drop postp1 postp2 postp3

getmata postp1
getmata postp2
getmata postp3

label variable postp1 "probability egalitarian"
label variable postp2 "probability libertarian"
label variable postp3 "probability lib-egalitarian"

scatter postp2 postp1 || function y=1-x, xtitle("probability egalitarian") ///
ytitle("probability libertarian") legend(off)


* 15.5 ESTIMATION OF SOCIAL PREFERENCE PARAMETERS USING DISCRETE CHOICE MODELS

use ES_sim, clear

asclogit y FSD FSA EFF MM, case( i) alternatives(j) noconstant

asclogit y  FSD male_FSD FSA male_FSA EFF MM  , case( i) alternatives(j) noconstant

* CHAPTER 16: REPEATED GAMES AND QUANTAL RESPONSE MODELS

* 16.2 ANALYSIS OF REPEATED GAME DATA

use pursue_evade, clear

* CHOICE FREQUENCIES BY PAIR

table pair, contents(mean pur_L mean eva_L)

* OVERALL CHOICE FREQUENCES

summ pur_L eva_L

* BINOMIAL TEST

bitest pur_L=0.6667 if pair==31
bitest eva_L=0.6667 if pair==31

* RUNS TEST

runtest pur_L in 1/100, t(0.5)
runtest eva_L in 1/100, t(0.5)

* 16.4 ESTIMATION OF THE QRE MODEL

*  Maximum likelihood estimation of "mu" in QRE model.  
* Simulated data generated within following code.  
* y1 is decision for PURSUER; y2 is decision for EVADER.
*  MATA Optimize routine is used to find p1 and p2 for given mu.
*  STATA ML routine maximises the log-likelihood over mu

clear
clear mata
set more off

* START MATA FROM WITHIN STATA

mata:

// SET STARTING VALUES FOR THE TWO PROBABILITIES

start=(0.5,0.5)

// CREATE PROGRAM ("vector_min") FOR EVALUATING 2x1 VECTOR (ss) 
// WHOSE ELEMENTS ARE TO BE MINIMISED 

void vector_min(todo, p, ss, S, H)
    {
		 external mu
                 p_PL   = p[1]
                 p_EL   = p[2]
                 
				 s1= (p_PL-exp(p_EL/mu)/(exp(p_EL/mu)+exp(2*(1-p_EL)/mu)))^2
				 s2= (p_EL-exp((-p_PL)/mu)/(exp((-p_PL)/mu)+exp(-2*(1-p_PL)/mu)))^2
				 
                 ss = s1 \ s2
  }  
         

// BEGIN DEFINITION OF OPTIMISATION PROBLEM, 
// RETURNING S, A PROBLEM-DESCRIPTION HANDLE CONTAINING DEFAULT VALUES		 
		 
 S = optimize_init()

// MODIFY DEFAULTS 
 
optimize_init_evaluator(S, &vector_min())
 optimize_init_evaluatortype(S, "gf0")
optimize_init_params(S, start)
 optimize_init_which(S,  "min" )
 optimize_init_tracelevel(S, "none")
optimize_init_conv_ptol(S, 1e-16)
 optimize_init_conv_vtol(S, 1e-16)
  
// RETURN TO STATA
 
end

clear

* GENERATE DATA SET

set obs 1000
gen int y_P = _n<501
gen int y_E = _n<668

* SET STARTING VALUES FOR COMPUTATION OF THE TWO PROBABILITIES

mat start=(0.67,0.67)

prog drop _all

* LOG-LIKELIHOOD EVALUATION PROGRAM (qre) STARTS HERE:

program define qre
	args lnf mu
	tempvar pp
	tempname p_PL p_EL
		
scalar mu=`mu'

* COPY STARTING PROBABILITIES AND mu INTO MATA
		
mata: start=st_matrix("start")	
mata: mu=st_numscalar("mu")

* PERFORM OPTIMIZATION; STORE SOLUTION IN 2x1 VECTOR p:

mata: p = optimize(S)

* EXTRACT ELEMENTS OF p

mata: st_numscalar("p_PL",p[1])
mata:  st_numscalar("p_EL", p[2])

* GENERATE JOINT PROBABILITY OF PURSUER'S AND EVADER'S DECISIONS:

	quietly gen double `pp'=((p_PL*y_P)+(1-p_PL)*(1-y_P))*((p_EL*y_E)+(1-p_EL)*(1-y_E))

* GENERATE LOG-LIKELIHOOD	
	
	quietly replace `lnf'=ln(`pp')
	end

* SET STARTING VALUE FOR ML ROUTINE (NOTE: ONLY ONE PARAMETER, mu)	
	
mat sstart=.6

* RUN ML
	
ml model lf qre ()
ml init sstart, copy
ml maximize


* 16.5 THE RISK-AVERSE QRE MODEL

*  Maximum likelihood estimation of "mu" in QRE model.  PURSUE-EVADE.  WITH r FOR RISK-AVERSION
*  Simulated data generated within code.  y1 is decision for PURSUER; y2 is decision for EVADER.
*  MATA Optimize routine is used to find p1 and p2 for given mu.
*  STATA ML routine maximises the log-likelihood over mu and r.

clear
clear mata
set more off

* START MATA FROM WITHIN STATA	
	
mata:

// SET STARTING VALUES FOR THE TWO PROBABILITIES

start=(0.5,0.5)

// CREATE PROGRAM ("vector_min") FOR EVALUATING 2x1 VECTOR (ss) 
// WHOSE ELEMENTS ARE TO BE MINIMISED 

void vector_min(todo, p, ss, S, H)
         {
		 external X, mu, r
                 PP   = p[1]
                 PE   = p[2]
                 
				 EU_PL=PE/(mu*(1-r))
				 EU_PR=(2^(1-r))*(1-PE)/(mu*(1-r))
				 
				 EU_EL=(-PP)/(mu*(1-r))
				 EU_ER=-(2^(1-r))*(1-PP)/(mu*(1-r))
				 
				 s1=(PP-exp(EU_PL)/(exp(EU_PL)+exp(EU_PR)))^2
				 s2=(PE-exp(EU_EL)/(exp(EU_EL)+exp(EU_ER)))^2 
				 
                 ss= s1\s2
		
         }

// BEGIN DEFINITION OF OPTIMISATION PROBLEM, 
// RETURNING S, A PROBLEM-DESCRIPTION HANDLE CONTAINING DEFAULT VALUES		 

 S = optimize_init()

// MODIFY DEFAULTS 

 
optimize_init_evaluator(S, &vector_min())
 optimize_init_evaluatortype(S, "v0")
optimize_init_params(S, start)
 optimize_init_which(S,  "min" )
 optimize_init_tracelevel(S,"none")
 optimize_init_conv_ptol(S, 1e-16)
 optimize_init_conv_vtol(S, 1e-16)
 
// RETURN TO STATA 
 
end

clear

* GENERATE DATA SET

set obs 1000
gen int y_P = _n<501
gen int y_E = _n<601

* SET STARTING VALUES FOR COMPUTATION OF THE TWO PROBABILITIES

mat start=(0.67,0.67)

prog drop _all

* LOG-LIKELIHOOD EVALUATION PROGRAM (qre_risk) STARTS HERE:

program define qre_risk
	args lnf mu r
	tempvar pp
	tempname p1 p2 mmu rr
		
scalar mmu=`mu'
scalar rr=`r'

* COPY STARTING PROBABILITIES AND TWO PARAMETERS (mu and r) INTO MATA
		
mata: start=st_matrix("start")	
mata: mu=st_numscalar("mmu")
mata: r=st_numscalar("rr")

* PERFORM OPTIMIZATION; STORE SOLUTION IN 2x1 VECTOR p:

mata: p = optimize(S)

* EXTRACT ELEMENTS OF p

mata: st_numscalar("p1",p[1])
mata:  st_numscalar("p2", p[2])

* GENERATE JOINT PROBABILITY OF PURSUER'S AND EVADER'S DECISIONS:

	quietly gen double `pp'=((p1*y_P)+(1-p1)*(1-y_P))*((p2*y_E)+(1-p2)*(1-y_E))

* GENERATE LOG-LIKELIHOOD	
	
	quietly replace `lnf'=ln(`pp')
	end

* SET STARTING VALUES FOR 2 PARAMETERS mu AND r	
	
mat sstart=(.1,0.3)

* RUN ML
	
ml model lf qre_risk () ()
ml init sstart, copy
ml maximize


* 16.6 QRE APPLIED TO CONTEST DATA

*  Maximum likelihood estimation of "mu" in QRE model for contest data.
*  MATA Optimize routine is used to find p0-p16 for given mu.
*  STATA ML routine maximises the log-likelihood over mu

clear
clear mata
set more off

mata:

start=(0.08,0.08,0.08,0.08,0.04,0.04,0.04,0.04,0.04,0.04, ///
0.04,0.04,0.04,0.04,0.04,0.04)

void mysolver(todo, p, fff, S, H)
    {
		 external mu
p1 = p[1]
p2 = p[2]
p3 = p[3]
p4 = p[4]
p5 = p[5]
p6 = p[6]
p7 = p[7]
p8 = p[8]
p9 = p[9]
p10 = p[10]
p11 = p[11]
p12 = p[12]
p13 = p[13]
p14 = p[14]
p15 = p[15]
p16 = p[16]
			 
p0=1-p1-p2-p3-p4-p5-p6-p7-p8-p9-p10-p11-p12-p13-p14-p15-p16
				 
ee=5*(0*p0+1*p1+2*p2+3*p3+4*p4+5*p5+6*p6+7*p7+8*p8+9*p9+10*p10  ///
 +11*p11+12*p12+13*p13+14*p14+15*p15+16*p16)
				  
				 				 
				 zz0=exp(0*(80/(0+3*ee)-1)/mu)
				  zz1=exp(5*(80/(5+3*ee)-1)/mu)
				   zz2=exp(10*(80/(10+3*ee)-1)/mu)
				    zz3=exp(15*(80/(15+3*ee)-1)/mu)
					 zz4=exp(20*(80/(20+3*ee)-1)/mu)
					  zz5=exp(25*(80/(25+3*ee)-1)/mu)
				 zz6=exp(30*(80/(30+3*ee)-1)/mu)
				  zz7=exp(35*(80/(35+3*ee)-1)/mu)
				   zz8=exp(40*(80/(40+3*ee)-1)/mu)
				    zz9=exp(45*(80/(45+3*ee)-1)/mu)
					 zz10=exp(50*(80/(50+3*ee)-1)/mu)
					
					zz11=exp(55*(80/(55+3*ee)-1)/mu)
				   zz12=exp(60*(80/(60+3*ee)-1)/mu)
				    zz13=exp(65*(80/(65+3*ee)-1)/mu)
					 zz14=exp(70*(80/(70+3*ee)-1)/mu)
					  zz15=exp(75*(80/(75+3*ee)-1)/mu)
				 zz16=exp(80*(80/(80+3*ee)-1)/mu)
				  
					
				zz=zz0+zz1+zz2+zz3+zz4+zz5+zz6+zz7+zz8+zz9+zz10+zz11 ///
				+ zz12+zz13+zz14+zz15+zz16
								
				pp0=zz0/zz
				pp1=zz1/zz
				pp2=zz2/zz
				pp3=zz3/zz
				pp4=zz4/zz
				pp5=zz5/zz
				pp6=zz6/zz
				pp7=zz7/zz
				pp8=zz8/zz
				pp9=zz9/zz
				pp10=zz10/zz
				
				pp11=zz11/zz
				pp12=zz12/zz
				pp13=zz13/zz
				pp14=zz14/zz
				pp15=zz15/zz
				pp16=zz16/zz
						
							
							
				fff=(p1-pp1)^2 \
				(p2-pp2)^2 \
				(p3-pp3)^2 \
				(p4-pp4)^2 \
				(p5-pp5)^2 \
				(p6-pp6)^2 \
				(p7-pp7)^2 \
				(p8-pp8)^2 \
				(p9-pp9)^2 \
				(p10-pp10)^2 \
					(p11-pp11)^2 \
				(p12-pp12)^2 \
				(p13-pp13)^2 \
				(p14-pp14)^2 \
				(p15-pp15)^2 \
				(p16-pp16)^2
				
					              
         }

 S = optimize_init()

optimize_init_evaluator(S, &mysolver())
 optimize_init_evaluatortype(S, "v0")
optimize_init_params(S, start)
 optimize_init_which(S,  "min" )
 optimize_init_tracelevel(S,"none")
 optimize_init_conv_ptol(S, 1e-16)
 optimize_init_conv_vtol(S, 1e-16)
 
end

clear

scalar mu=10.0

mat start=(1.101704)

prog drop _all

	mata: start=st_matrix("start")	

mata: mu=st_numscalar("mu")


mata: p = optimize(S)

mata: start=p

mata: st_numscalar("p1",p[1])
mata: st_numscalar("p2",p[2])
mata: st_numscalar("p3",p[3])
mata: st_numscalar("p4",p[4])
mata: st_numscalar("p5",p[5])
mata: st_numscalar("p6",p[6])
mata: st_numscalar("p7",p[7])
mata: st_numscalar("p8",p[8])
mata: st_numscalar("p9",p[9])
mata: st_numscalar("p10",p[10])

mata: st_numscalar("p11",p[11])
mata: st_numscalar("p12",p[12])
mata: st_numscalar("p13",p[13])
mata: st_numscalar("p14",p[14])
mata: st_numscalar("p15",p[15])
mata: st_numscalar("p16",p[16])


scalar p0=1-p1-p2-p3-p4-p5-p6-p7-p8-p9-p10-p11-p12-p13-p14-p15-p16

clear

mat start=(0.08,0.08,0.08,0.08,0.04,0.04,0.04, ///
0.04,0.04,0.04,0.04,0.04,0.04,0.04,0.04,0.04)

prog drop _all

program define qre
	args lnf mu 
	tempvar pp
	tempname p0 p1 p2 p3 p4 p5 p6 p7 p8 p9 p10 p11 p12 p13 p14 p15 p16 mmu
		
scalar mmu=`mu'
		
mata: start=st_matrix("start")	

mata: mu=st_numscalar("mmu")
mata: X=st_numscalar("X")

mata: p = optimize(S)

mata: start=p

mata: st_numscalar("p1",p[1])
mata:  st_numscalar("p2", p[2])
mata: st_numscalar("p3",p[3])
mata:  st_numscalar("p4", p[4])
mata: st_numscalar("p5",p[5])
mata:  st_numscalar("p6", p[6])		
mata:  st_numscalar("p7", p[7])
mata: st_numscalar("p8",p[8])
mata:  st_numscalar("p9", p[9])	
mata:  st_numscalar("p10", p[10])	
mata: st_numscalar("p11",p[11])
mata:  st_numscalar("p12", p[12])
mata: st_numscalar("p13",p[13])
mata:  st_numscalar("p14", p[14])
mata: st_numscalar("p15",p[15])
mata:  st_numscalar("p16", p[16])		
		
scalar p0=1-p1-p2-p3-p4-p5-p6-p7-p8-p9-p10-p11-p12-p13-p14-p15-p16

forvalues k=1(1)16 {
scalar p`k'=max(p`k',1.0e-12)
}

scalar p0=1-p1-p2-p3-p4-p5-p6-p7-p8-p9-p10-p11-p12-p13-p14-p15-p16

	quietly gen double `pp'=y0*p0+y1*p1+y2*p2+y3*p3+y4*p4+y5*p5+y6*p6 ///
  +y7*p7+y8*p8+y9*p9+y10*p10+y11*p11+y12*p12+y13*p13+y14*p14+y15*p15+y16*p16
		
	quietly replace `lnf'=ln(`pp')
	end

use chowdhury, clear

keep if t>15

* ROUND BID TO NEAREST 5

gen y=5*round(bid/5,1)

gen y0=y==0
gen y1=y==5
gen y2=y==10
gen y3=y==15
gen y4=y==20
gen y5=y==25
gen y6=y==30
gen y7=y==35
gen y8=y==40
gen y9=y==45
gen y10=y==50
gen y11=y==55
gen y12=y==60
gen y13=y==65
gen y14=y==70
gen y15=y==75
gen y16=y==80
	
mat sstart=(10)
	
ml model lf qre ()
ml init sstart, copy
ml maximize


* CHAPTER 17: DEPTH OF REASONING MODELS

*  17.2 LEVEL-K MODEL FOR THE BEAUTY CONTEST GAME

clear
program drop _all
set more off

* LIKELIHOOD EVALUATION PROGRAM STARTS HERE:

program define beauty_mixture
	args logl p1 p2 p3 p4 p5 sig
	tempvar  f0 f1 f2 f3 f4 f5 l

	quietly{
	
	gen double `f0'=0.01
	gen double `f1'=(1/`sig')*normalden((y-33.5)/`sig')
	gen double `f2'=(1/`sig')*normalden((y-22.4)/`sig')
	gen double `f3'=(1/`sig')*normalden((y-15.0)/`sig')
	gen double `f4'=(1/`sig')*normalden((y-10.1)/`sig')
	gen double `f5'=(1/`sig')*normalden((y-0)/`sig')
	
	gen double `l'=(1-`p1'-`p2'-`p3'-`p4'-`p5')*`f0'   ///
	+`p1'*`f1'+`p2'*`f2'+`p3'*`f3'+`p4'*`f4'+`p5'*`f5'
	
	replace postp0=(1-`p1'-`p2'-`p3'-`p4'-`p5')*`f0'/`l'
	replace postp1=`p1'*`f1'/`l'
	replace postp2=`p2'*`f2'/`l'
	replace postp3=`p3'*`f3'/`l'
	replace postp4=`p4'*`f4'/`l'
	replace postp5=`p5'*`f5'/`l'
	
	replace `logl'=ln(`l')
		
 putmata postp0, replace
 putmata postp1, replace
 putmata postp2, replace
 putmata postp3, replace
 putmata postp4, replace
 putmata postp5, replace
	
	}
	
	end

* END OF LIKELIHOOD EVALUATION PROGRAM

* READ DATA	
	
use beauty_sim, clear	

hist y, disc xtitle("guess")

* INITIALISE POSTERIOR TYPE PROBABILITIES
	
gen postp0=.
gen postp1=.
gen postp2=.
gen postp3=.
gen postp4=.
gen postp5=.

* SET STARTING VALUES AND RUN ML

mat start=(0.3, 0.4, 0.1, 0.1,0.05, 2)
ml model lf beauty_mixture /p1 /p2 /p3 /p4 /p5 /sig
ml init start, copy
ml maximize

* DEDUCE PROBABILITY OF LEVEL-0 TYPE

nlcom p0: 1-_b[p1:_cons]-_b[p2:_cons]-_b[p3:_cons]-_b[p4:_cons]-_b[p5:_cons]

* EXTRACT POSTERIOR TYPE PROBABILITIES

drop postp*
getmata postp0
getmata postp1
getmata postp2
getmata postp3
getmata postp4
getmata postp5

* PLOT TYPE PROBABILITIES AGAINST GUESS

sort y

line  postp0 postp1 postp2 postp3 postp4 postp5 y , lpattern(- l l l l l) ///
 xline(21.7) xtitle("guess") ytitle("posterior type probability") legend(off)

*  17.3 THE COGNITIVE HIERARCHY MODEL


program drop _all

*Log-likelihood evaluation program (cog_hier) starts here

program define cog_hier
	args logl sig tau
	tempvar  f0 f1 f2 f3 f4 f5 l
	tempname p0 p1 p2 p3 p4 p5 b0 b1 b2 b3 b4 b5 

	scalar `p0'=exp(-`tau')
scalar `p1'=`p0'*`tau'/1
scalar `p2'=`p1'*`tau'/2
scalar `p3'=`p2'*`tau'/3
scalar `p4'=`p3'*`tau'/4
scalar `p5'=1-`p0'-`p1'-`p2'-`p3'-`p4'
	
	scalar `b0'=50
	scalar `b1'=.67*`b0'
scalar `b2'=.67*(`p1'*`b1'+`p0'*`b0')/(`p1'+`p0')
scalar `b3'=.67*(`p2'*`b2'+`p1'*`b1'+`p0'*`b0')/(`p2'+`p1'+`p0')
scalar `b4'=.67*(`p3'*`b3'+`p2'*`b2'+`p1'*`b1'+`p0'*`b0')/(`p3'+`p2'+`p1'+`p0')
	
	quietly{
	
	gen double `f0'=0.01
	gen double `f1'=(1/`sig')*normalden((y-`b1')/`sig')
	gen double `f2'=(1/`sig')*normalden((y-`b2')/`sig')
	gen double `f3'=(1/`sig')*normalden((y-`b3')/`sig')
	gen double `f4'=(1/`sig')*normalden((y-`b4')/`sig')
	gen double `f5'=(1/`sig')*normalden((y-0)/`sig')
	
	gen double `l'=`p0'*`f0'+`p1'*`f1'+`p2'*`f2'+`p3'*`f3'+`p4'*`f4'+`p5'*`f5'
	
	replace `logl'=ln(`l')
	
	replace postp0=`p0'*`f0'/`l'
	replace postp1=`p1'*`f1'/`l'
	replace postp2=`p2'*`f2'/`l'
	replace postp3=`p3'*`f3'/`l'
	replace postp4=`p4'*`f4'/`l'
	replace postp5=`p5'*`f5'/`l'	
	
	putmata postp0, replace
 putmata postp1, replace
 putmata postp2, replace
 putmata postp3, replace
 putmata postp4, replace
 putmata postp5, replace
	}
	
	end

* READ DATA

use cog_hier_sim, clear	
	
* create posterior prob variables, set starting values and call ML program (ch)
	
gen postp0=.
gen postp1=.
gen postp2=.
gen postp3=.
gen postp4=.
gen postp5=.

	
mat start=( 2,2)
ml model lf cog_hier /sig /tau
ml init start, copy

ml maximize

drop postp*

getmata postp0
getmata postp1
getmata postp2
getmata postp3
getmata postp4
getmata postp5

sort y

line  postp0 postp1 postp2 postp3 postp4 postp5 y , lpattern(- l l l l l) ///
legend(off) xlabel(0(10)100) xtitle(guess) ytitle("posterior type probability") ///
xline(18.79)

* CHAPTER 18: LEARNING MODELS

* 18.2 DIRECTIONAL LEARNING (DL)

use selten_stoecker, clear

xtset i t

regress d.self l.before l.same l.after, nocon


* 18.5 REINFORCEMENT LEARNING (RL)


set more off
program drop _all

* LIKELIHOOD EVALUATION PROGRAM STARTS HERE
	
program define reinforcement
	
* SPECIFY ARGUMENTS: NAMES OF MAXIMAND AND 4 PARAMETERS
	
	args logl phi lam A10_start A20_start 	
	
quietly{

* INITIALISE ATTRACTION VARIABLES FOR CURRENT LIKELIHOOD EVALUATION:

replace A10=.
replace A11=.
replace A20=.
replace A21=.

* UPDATE ATTRACTIONS BY ADDING PAY-OFFS FROM CHOSEN STRATEGIES
* Aij IS PLAYER i's ATTRACTION TO STRATEGY j, 
* UPDATED BY ACTUAL PAYOFF IN CURRENT PERIOD,
* FIRST GENERATE VALUES OF ATTRACTION VARIABLES IN PERIOD 1 
* (USING INITIAL ATTRACTIONS)
* THEN GENERATE VALUES OF ATTRACTION VARIABLES IN SUBSEQUENT PERIODS
	
by i: replace A10=`phi'*`A10_start'+wx10 if _n==1
by i: replace A11=`phi'*0+wx11 if _n==1
by i: replace A20=`phi'*`A20_start'+wx20 if _n==1
by i: replace A21=`phi'*0+wx21 if _n==1
				
by i: replace A11=`phi'*A11[_n-1]+wx11    if A11==.
by i: replace A10=`phi'*A10[_n-1]+wx10    if A10==.
by i: replace A21=`phi'*A21[_n-1]+wx21    if A21==.
by i: replace A20=`phi'*A20[_n-1]+wx20    if A20==.
		
* GENERATE PROBABILITY OF PLAYER i CHOOSING STRATEGY j (pij) 
* USING _PREVIOUS_ PERIOD'S ATTRACTIONS
				
replace p11=.
replace p21=.
				
by i: replace p11=exp(`lam'*0)/(exp(`lam'*0)+exp(`lam'*`A10_start')) if _n==1  
by i: replace p21=exp(`lam'*0)/(exp(`lam'*0)+exp(`lam'*`A20_start')) if _n==1
				
by i: replace p11=exp(`lam'*A11[_n-1])/(exp(`lam'*A11[_n-1]) ///
+exp(`lam'*A10[_n-1])) if p11==. 
by i: replace p21=exp(`lam'*A21[_n-1])/(exp(`lam'*A21[_n-1]) ///
+exp(`lam'*A20[_n-1])) if p21==.

* GENERATE LOG-LIKELIHOOD CONTRIBUTION AS THE PRODUCT OF THE PROBABILITIES 
* OF THE CHOICES OF THE TWO PLAYERS 	
	
	quietly replace `logl'=ln((p11*y1+(1-p11)*(1-y1))*(p21*y2+(1-p21)*(1-y2)))				
	}	
	end

* LIKELIHOOD EVALUATION PROGRAM ENDS HERE

* READ DATA

use "pursue_evade_sim.dta", clear

* GENERATE AMOUNT EACH PLAYER _WOULD_ RECEIVE BY PLAYING EACH STRATEGY,
* _GIVEN_ THE STRATEGY CHOSEN BY THE OTHER PLAYER
* x_ij IS PAYOFF PLAYER i (i=1,2) WOULD RECEIVE BY PLAYING STRATEGY j 
* j=0,1; 0=LEFT; 1=RIGHT).

		gen int x11= 2*(y2==1)+0*(y2==0)   
		gen int x10= 0*(y2==1)+1*(y2==0)   
		
		gen int x21= (-2)*(y1==1)+0*(y1==0)  
		gen int x20= 0*(y1==1)+(-1)*(y1==0)  

* GENERATE AMOUNT EACH PLAYER RECIEVES BY PLAYING THE STRATEGY THEY CHOOSE;
* ZERO FOR THE UNCHOSEN STRATEGY
* wx_ij IS AMOUNT RECEIEVED BY i CHOOSING j.  wx_ij = 0 IF j NOT CHOSEN.
				
		gen int wx11= (y1)*x11
		gen int wx10= (1-y1)*x10
				
		gen int wx21= (y2)*x21
		gen int wx20= (1-y2)*x20			

* INITIALISE ATTRACTION VARIABLES, AND CHOICE PROBABILITY VARIABLES
				
		gen double A10=.
		gen double A11=.
		gen double A20=.
		gen double A21=.
	
		gen double p11=.
		gen double p21=.
		
* SET STARTING VALUES:

mat start=( 0.95,0.20,0.0,0.0)

*RUN ML

ml model lf reinforcement  /phi  /lam /A10_start /A20_start 
ml init start, copy

ml max, trace search(norescale)


* 18.8 BELIEF LEARNING (BL)


set more off
program drop _all

* LIKELIHOOD EVALUATION PROGRAM STARTS HERE

program define belief
	args logl phi lam A10_start A20_start N_start
	
	
quietly{

replace A10=.
replace A11=.
replace A20=.
replace A21=.

replace N=.

by i: replace N=`phi'*`N_start'+1 if _n==1
by i: replace N=`phi'*N[_n-1]+1 if N==.	
	
by i: replace A10=(`phi'*`N_start'*`A10_start'+x10)/N if _n==1
by i: replace A11=(`phi'*`N_start'*0+x11)/N if _n==1
by i: replace A20=(`phi'*`N_start'*`A20_start'+x20)/N if _n==1
by i: replace A21=(`phi'*`N_start'*0+x21)/N if _n==1
			 				
* Aij is the attraction, updated by payoff (either actual or hypothetical) in t, 
* to be used to determine choice probs in t+1
				
	by i: replace A11=(`phi'*N[_n-1]*A11[_n-1]+x11)/N    if A11==.
	by i: replace A10=(`phi'*N[_n-1]*A10[_n-1]+x10)/N    if A10==.
	by i: replace A21=(`phi'*N[_n-1]*A21[_n-1]+x21)/N    if A21==.
	by i: replace A20=(`phi'*N[_n-1]*A20[_n-1]+x20)/N    if A20==.
				
	*pij are the probabilities player i choosing strategy j
				
	replace p11=.
	replace p21=.
				
	by i: replace p11=exp(`lam'*0)/(exp(`lam'*0)+exp(`lam'*`A10_start')) if _n==1  
	by i: replace p21=exp(`lam'*0)/(exp(`lam'*0)+exp(`lam'*`A20_start')) if _n==1
				
	by i: replace p11=exp(`lam'*A11[_n-1])/(exp(`lam'*A11[_n-1]) ///
	+exp(`lam'*A10[_n-1])) if p11==. 
	by i: replace p21=exp(`lam'*A21[_n-1])/(exp(`lam'*A21[_n-1]) ///
	+exp(`lam'*A20[_n-1])) if p21==.
						
	replace `logl'=ln((p11*y1+(1-p11)*(1-y1))*(p21*y2+(1-p21)*(1-y2)))						
			}			
						
	
	end

* LIKELIHOOD EVALUATION PROGRAM ENDS HERE

* READ DATA

use "pursue_evade_sim.dta", clear

* GENERATE AMOUNT EACH PLAYER _WOULD_ RECEIVE BY PLAYING EACH STRATEGY,
* _GIVEN_ THE STRATEGY CHOSEN BY THE OTHER PLAYER
* x_ij IS PAYOFF PLAYER i (i=1,2) RECEIVES BY PLAYING STRATEGY j 
* (j=0,1; 0=LEFT; 1=RIGHT).

		gen int x11= 2*(y2==1)+0*(y2==0)   
		gen int x10= 0*(y2==1)+1*(y2==0)   
		gen int x21= (-2)*(y1==1)+0*(y1==0)  
		gen int x20= 0*(y1==1)+(-1)*(y1==0)  

* INITIALISE OTHER VARIABLES
		
		gen double A10=.
		gen double A11=.
		gen double A20=.
		gen double A21=.
		
		gen double N=.

		gen double wx11=.
		gen double wx10=.
		gen double wx21=.
		gen double wx20=.
		
		gen double p11=.
		gen double p21=.
			
* STARTING VALUES:

mat start=( 0.95,0.20,0.0,0.0,1.0)

*RUNNING ML

ml model lf belief  /phi /lambda /A10_start /A20_start /N_start
ml init start, copy
ml max, trace search(norescale)


* 18.7 THE EXPERIENCE WEIGHTED ATTRACTION (EWA) MODEL


set more off
program drop _all

* LIKELIHOOD EVALUATION PROGRAM STARTS HERE

program define ewa
	args lnf rho delta phi lambda A10_start A20_start N_start
		
	tempvar  
	tempname 
	
quietly{

replace A10=.
replace A11=.
replace A20=.
replace A21=.

* GENERATE EXPERIENCE VARIABLE, N(t), STARTING WITH PERIOD 1:

replace N=.
by i: replace N=`rho'*`N_start'+1 if _n==1
by i: replace N=`rho'*N[_n-1]+1 if N==.	
	
* GENERATE wxij (PAY-OFFS WEIGHTED BY DELTA PARAMETER): 
				
				replace wx11= (`delta'+(1-`delta')*(y1))*x11
				replace wx10= (`delta'+(1-`delta')*(1-y1))*x10
				replace wx21= (`delta'+(1-`delta')*(y2))*x21
				replace wx20= (`delta'+(1-`delta')*(1-y2))*x20	
 				
* GENERATE PERIOD-1 ATTRACTIONS:
		
		by i: replace A10=(`phi'*`N_start'*`A10_start'+wx10)/N if _n==1
		by i: replace A11=(`phi'*`N_start'*0+wx11)/N if _n==1
		by i: replace A20=(`phi'*`N_start'*`A20_start'+wx20)/N if _n==1
		by i: replace A21=(`phi'*`N_start'*0+wx21)/N if _n==1
		
* GENERATE ATTRACTIONS FOR t>1:		
		
		by i: replace A11=(`phi'*N[_n-1]*A11[_n-1]+wx11)/N    if A11==.
		by i: replace A10=(`phi'*N[_n-1]*A10[_n-1]+wx10)/N    if A10==.
		by i: replace A21=(`phi'*N[_n-1]*A21[_n-1]+wx21)/N    if A21==.
		by i: replace A20=(`phi'*N[_n-1]*A20[_n-1]+wx20)/N    if A20==.
				
* GENERATE p11 AND p21 (PROBABILITIES OF PLAYERS 1 and 2 CHOOSING STRATEGY 1):
				
		replace p11=.
		replace p21=.
				
		by i: replace p11=exp(`lambda'*0)/(exp(`lambda'*0) ///
		+exp(`lambda'*`A10_start')) if _n==1  
		by i: replace p21=exp(`lambda'*0)/(exp(`lambda'*0) ///
		+exp(`lambda'*`A20_start')) if _n==1
				
		by i: replace p11=exp(`lambda'*A11[_n-1])/(exp(`lambda'*A11[_n-1]) ///
		+exp(`lambda'*A10[_n-1])) if p11==. 
		by i: replace p21=exp(`lambda'*A21[_n-1])/(exp(`lambda'*A21[_n-1]) ///
		+exp(`lambda'*A20[_n-1])) if p21==.

* GENERATE LOG-LIKELIHOOD		
		
		replace `lnf'=ln((p11*y1+(1-p11)*(1-y1))*(p21*y2+(1-p21)*(1-y2)))				
		}
	end

* LIKELIHOOD EVALUATION PROGRAM ENDS HERE

* READ DATA

use "pursue_evade_sim.dta", clear

* GENERATE PAY-OFFS (xij) THAT PLAYER i WOULD RECIEVE FROM CHOOSING STRATEGY j.

		gen int x11= 2*(y2==1)+0*(y2==0)   
		gen int x10= 0*(y2==1)+1*(y2==0)   
		
		gen int x21= (-2)*(y1==1)+0*(y1==0)  
		gen int x20= 0*(y1==1)+(-1)*(y1==0)  

* INITIALISE VARIOUS VARIABLES USED WITHIN PROGRAM:		
		
		gen double A10=.
		gen double A11=.
		gen double A20=.
		gen double A21=.
		
		gen double N=.

		gen double wx11=.
		gen double wx10=.
		gen double wx21=.
		gen double wx20=.
		
		gen double p11=.
		gen double p21=.
			
* SET STARTING VALUES:

mat start=( 0.993, 0.78,0.998,0.7531,0.657,-1.863,0.833)

*RUNNING ML: FULL EWA MODEL

ml model lf ewa /rho /delta /phi  /lambda /A10_start /A20_start /N_start
ml init start, copy
ml max, trace search(norescale)
est store ewa

* DEFINE CONSTRAINTS REQUIRED FOR RL AND BL:

constraint 1 [delta]_b[_cons]=0.0
constraint 2 [rho]_b[_cons]=0.0
constraint 3 [delta]_b[_cons]=1
constraint 4 [rho]_b[_cons]=[phi]_b[_cons]
constraint 5 [N_start]_b[_cons]=1

*  ESTIMATE RL AS RESTRICTED VERSION OF EWA:

ml model lf ewa /rho /delta /phi  /lambda /A10_start /A20_start /N_start, ///
 constraints(1 2 5)
ml init start, copy
ml max, trace search(norescale)
est store rl

*  ESTIMATE BL AS RESTRICTED VERSION OF EWA:

ml model lf ewa /rho /delta /phi  /lambda /A10_start /A20_start /N_start, ///
 constraints(3 4)
ml init start, copy
ml max, trace search(norescale)
est store bl

* LR TESTS FOR RL AND BL AS RESTRICTIONS OF EWA:

lrtest ewa rl
lrtest ewa bl



* END OF DO-FILE
